@@ -3,6 +3,7 @@
 # Use `test/fakesmsc' to test the bearerbox and the smsbox.
 
 set -e
+#set -x
 
 times=10
 interval=0
@@ -11,27 +12,26 @@ host=127.0.0.1
 
 gw/bearerbox -v $loglevel gw/smskannel.conf > check_httpsmsc_kannel_sbb.log 2>&1 &
 sbbpid=$!
+sleep 1
 
 gw/bearerbox -v $loglevel gw/other_smskannel.conf > check_httpsmsc_kannel_cbb.log 2>&1 &
 cbbpid=$!
-
 sleep 2
 
 test/fakesmsc -H $host -i $interval -m $times '123 234 text relay nop' \
     > check_httpsmsc_kannel_fake.log 2>&1 &
-
 sleep 1
 
 gw/smsbox -v $loglevel gw/smskannel.conf > check_httpsmsc_kannel_ssb.log 2>&1 &
 gw/smsbox -v $loglevel gw/other_smskannel.conf > check_httpsmsc_kannel_csb.log 2>&1 &
 
-running=yes
-while [ $running = yes ]
+running="yes"
+while [ $running = "yes" ]
 do
     sleep 1
     if grep -v "fakesmsc: terminating" check_httpsmsc_kannel_fake.log >/dev/null
     then
-    	running=no
+    	running="no"
     fi
 done
 
@@ -46,4 +46,4 @@ then
 	exit 1
 fi
 
-rm check_httpsmsc_kannel_*.log
+rm -f check_httpsmsc_kannel_*.log
