@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -266,6 +266,42 @@ static Octstr *httpd_stop_smsc(List *cgivars, int status_type)
         return octstr_create("SMSC id not given");
 }
 
+static Octstr *httpd_remove_smsc(List *cgivars, int status_type)
+{
+    Octstr *reply;
+    Octstr *smsc;
+    if ((reply = httpd_check_authorization(cgivars, 0))!= NULL) return reply;
+    if ((reply = httpd_check_status())!= NULL) return reply;
+
+    /* check if the smsc id is given */
+    smsc = http_cgi_variable(cgivars, "smsc");
+    if (smsc) {
+        if (bb_remove_smsc(smsc) == -1)
+            return octstr_format("Could not remove smsc-id `%s'", octstr_get_cstr(smsc));
+        else
+            return octstr_format("SMSC `%s' removed", octstr_get_cstr(smsc));
+    } else
+        return octstr_create("SMSC id not given");
+}
+
+static Octstr *httpd_add_smsc(List *cgivars, int status_type)
+{
+    Octstr *reply;
+    Octstr *smsc;
+    if ((reply = httpd_check_authorization(cgivars, 0))!= NULL) return reply;
+    if ((reply = httpd_check_status())!= NULL) return reply;
+
+    /* check if the smsc id is given */
+    smsc = http_cgi_variable(cgivars, "smsc");
+    if (smsc) {
+        if (bb_add_smsc(smsc) == -1)
+            return octstr_format("Could not add smsc-id `%s'", octstr_get_cstr(smsc));
+        else
+            return octstr_format("SMSC `%s' added", octstr_get_cstr(smsc));
+    } else
+        return octstr_create("SMSC id not given");
+}
+
 static Octstr *httpd_restart_smsc(List *cgivars, int status_type)
 {
     Octstr *reply;
@@ -296,6 +332,26 @@ static Octstr *httpd_reload_lists(List *cgivars, int status_type)
         return octstr_create("Black/white lists re-loaded");
 }
 
+static Octstr *httpd_remove_message(List *cgivars, int status_type)
+{
+    Octstr *reply;
+    Octstr *message_id;
+    if ((reply = httpd_check_authorization(cgivars, 0))!= NULL) return reply;
+    if ((reply = httpd_check_status())!= NULL) return reply;
+
+    /* check if the smsc id is given */
+    message_id = http_cgi_variable(cgivars, "id");
+    if (message_id) {
+        if (octstr_len(message_id) != UUID_STR_LEN)
+            return octstr_format("Message id should be %d characters long", UUID_STR_LEN);
+        if (bb_remove_message(message_id) == -1)
+            return octstr_format("Could not remove message id `%s'", octstr_get_cstr(message_id));
+        else
+            return octstr_format("Message id `%s' removed", octstr_get_cstr(message_id));
+    } else
+        return octstr_create("Message id not given");
+}
+
 /* Known httpd commands and their functions */
 static struct httpd_command {
     const char *command;
@@ -312,7 +368,10 @@ static struct httpd_command {
     { "flush-dlr", httpd_flush_dlr },
     { "stop-smsc", httpd_stop_smsc },
     { "start-smsc", httpd_restart_smsc },
+    { "add-smsc", httpd_add_smsc },
+    { "remove-smsc", httpd_remove_smsc },
     { "reload-lists", httpd_reload_lists },
+    { "remove-message", httpd_remove_message },
     { NULL , NULL } /* terminate list */
 };
 
