@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -112,7 +112,8 @@ enum {
     SMSCCONN_FAILED_MALFORMED,
     SMSCCONN_FAILED_TEMPORARILY,
     SMSCCONN_FAILED_DISCARDED,
-    SMSCCONN_FAILED_QFULL
+    SMSCCONN_FAILED_QFULL,
+    SMSCCONN_FAILED_EXPIRED
 };
 
 
