@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -75,6 +75,8 @@
  *     instance. This is required if you use state conditioned smsboxes or smppboxes
  *     via one bearerbox. Previously bearerbox was simple ignoring to which smsbox
  *     connection a msg is passed. Now we can route the messages inside bearerbox.
+ * 2009-04-29: aguerrieri at kannel dot org:
+ *     added support for ms-sql and sybase via freetds.
  */
  
 #include <ctype.h>
@@ -95,7 +97,7 @@ static struct dlr_storage *handles = NULL;
 
 /*
  * Function to allocate a new struct dlr_entry entry
- * and intialize it to zero
+ * and initialize it to zero
  */
 struct dlr_entry *dlr_entry_create(void)
 {
@@ -173,6 +175,9 @@ struct dlr_db_fields *dlr_db_fields_create(CfgGroup *grp)
 
     if (!(ret->table = cfg_get(grp, octstr_imm("table"))))
    	    panic(0, "DLR: DB: directive 'table' is not specified!");
+    ret->ttl = 0;
+    if (cfg_get_integer(&ret->ttl, grp, octstr_imm("ttl")) == -1 || ret->ttl < 0)
+        ret->ttl = 0;
     if (!(ret->field_smsc = cfg_get(grp, octstr_imm("field-smsc"))))
    	    panic(0, "DLR: DB: directive 'field-smsc' is not specified!");
     if (!(ret->field_ts = cfg_get(grp, octstr_imm("field-timestamp"))))
@@ -246,7 +251,9 @@ void dlr_init(Cfg* cfg)
     }
 
     /* call the sub-init routine */
-    if (octstr_compare(dlr_type, octstr_imm("mysql")) == 0) {
+    if (octstr_compare(dlr_type, octstr_imm("spool")) == 0) {
+        handles = dlr_init_spool(cfg);
+    } else if (octstr_compare(dlr_type, octstr_imm("mysql")) == 0) {
         handles = dlr_init_mysql(cfg);
     } else if (octstr_compare(dlr_type, octstr_imm("sdb")) == 0) {
         handles = dlr_init_sdb(cfg);
@@ -256,6 +263,12 @@ void dlr_init(Cfg* cfg)
         handles = dlr_init_mem(cfg);
     } else if (octstr_compare(dlr_type, octstr_imm("pgsql")) == 0) {
         handles = dlr_init_pgsql(cfg);
+    } else if (octstr_compare(dlr_type, octstr_imm("mssql")) == 0) {
+        handles = dlr_init_mssql(cfg);
+    } else if (octstr_compare(dlr_type, octstr_imm("sqlite3")) == 0) {
+        handles = dlr_init_sqlite3(cfg);
+    } else if (octstr_compare(dlr_type, octstr_imm("redis")) == 0) {
+        handles = dlr_init_redis(cfg);
     }
 
     /*
@@ -309,19 +322,26 @@ const char* dlr_type(void)
 }
  
 /*
- * Add new dlr entry into dlr storage
+ * Add new dlr entry into dlr storage.
  */
-void dlr_add(const Octstr *smsc, const Octstr *ts, Msg *msg)
+void dlr_add(const Octstr *smsc, const Octstr *ts, Msg *msg, int use_dst)
 {
     struct dlr_entry *dlr = NULL;
 
-    /* Add the foreign_id so all SMSC modules can use it */
+    /* Add the foreign_id so all SMSC modules can use it.
+     * Obey also the original message in the split_parts list. */
     if (msg->sms.foreign_id != NULL)
         octstr_destroy(msg->sms.foreign_id);
     msg->sms.foreign_id = octstr_duplicate(ts);
+    if (msg->sms.split_parts != NULL) {
+        struct split_parts *split = msg->sms.split_parts;
+        if (split->orig->sms.foreign_id != NULL)
+            octstr_destroy(split->orig->sms.foreign_id);
+        split->orig->sms.foreign_id = octstr_duplicate(ts);
+    }
 
-    if(octstr_len(smsc) == 0) {
-	warning(0, "DLR[%s]: Can't add a dlr without smsc-id", dlr_type());
+    if (octstr_len(smsc) == 0) {
+        warning(0, "DLR[%s]: Can't add a dlr without smsc-id", dlr_type());
         return;
     }
 
@@ -346,10 +366,11 @@ void dlr_add(const Octstr *smsc, const Octstr *ts, Msg *msg)
     dlr->url = (msg->sms.dlr_url ? octstr_duplicate(msg->sms.dlr_url) : octstr_create(""));
     dlr->boxc_id = (msg->sms.boxc_id ? octstr_duplicate(msg->sms.boxc_id) : octstr_create(""));
     dlr->mask = msg->sms.dlr_mask;
+    dlr->use_dst = use_dst;
 
     debug("dlr.dlr", 0, "DLR[%s]: Adding DLR smsc=%s, ts=%s, src=%s, dst=%s, mask=%d, boxc=%s",
-              dlr_type(), octstr_get_cstr(dlr->smsc), octstr_get_cstr(dlr->timestamp),
-              octstr_get_cstr(dlr->source), octstr_get_cstr(dlr->destination), dlr->mask, octstr_get_cstr(dlr->boxc_id));
+          dlr_type(), octstr_get_cstr(dlr->smsc), octstr_get_cstr(dlr->timestamp),
+          octstr_get_cstr(dlr->source), octstr_get_cstr(dlr->destination), dlr->mask, octstr_get_cstr(dlr->boxc_id));
 	
     /* call registered function */
     handles->dlr_add(dlr);
@@ -360,10 +381,11 @@ void dlr_add(const Octstr *smsc, const Octstr *ts, Msg *msg)
  * NOTE: If typ is end status (e.g. DELIVERED) then dlr entry
  *       will be removed from DB.
  */
-Msg *dlr_find(const Octstr *smsc, const Octstr *ts, const Octstr *dst, int typ)
+Msg *dlr_find(const Octstr *smsc, const Octstr *ts, const Octstr *dst, int typ, int use_dst)
 {
     Msg	*msg = NULL;
     struct dlr_entry *dlr = NULL;
+    Octstr *dst_min = NULL;
     
     if(octstr_len(smsc) == 0) {
 	warning(0, "DLR[%s]: Can't find a dlr without smsc-id", dlr_type());
@@ -374,10 +396,17 @@ Msg *dlr_find(const Octstr *smsc, const Octstr *ts, const Octstr *dst, int typ)
     if (handles == NULL || handles->dlr_get == NULL)
         return NULL;
 
+    if (use_dst && dst) {
+        dst_min = octstr_duplicate(dst);
+        int len = octstr_len(dst);
+
+        if (len > MIN_DST_LEN)
+            octstr_delete(dst_min, 0, len - MIN_DST_LEN);
+    }
     debug("dlr.dlr", 0, "DLR[%s]: Looking for DLR smsc=%s, ts=%s, dst=%s, type=%d",
                                  dlr_type(), octstr_get_cstr(smsc), octstr_get_cstr(ts), octstr_get_cstr(dst), typ);
 
-    dlr = handles->dlr_get(smsc, ts, dst);
+    dlr = handles->dlr_get(smsc, ts, dst_min);
     if (dlr == NULL)  {
         warning(0, "DLR[%s]: DLR from SMSC<%s> for DST<%s> not found.",
                 dlr_type(), octstr_get_cstr(smsc), octstr_get_cstr(dst));         
@@ -425,12 +454,13 @@ Msg *dlr_find(const Octstr *smsc, const Octstr *ts, const Octstr *dst, int typ)
     if ((typ & DLR_BUFFERED) && ((dlr->mask & DLR_SUCCESS) || (dlr->mask & DLR_FAIL))) {
         debug("dlr.dlr", 0, "DLR[%s]: DLR not destroyed, still waiting for other delivery report", dlr_type());
         /* update dlr entry status if function defined */
-        if (handles != NULL && handles->dlr_update != NULL)
-            handles->dlr_update(smsc, ts, dst, typ);
+        if (handles != NULL && handles->dlr_update != NULL){
+            handles->dlr_update(smsc, ts, dst_min, typ);
+        }
     } else {
-        if (handles != NULL && handles->dlr_remove != NULL) {
+        if (handles != NULL && handles->dlr_remove != NULL){
             /* it's not good for internal storage, but better for all others */
-            handles->dlr_remove(smsc, ts, dst);
+            handles->dlr_remove(smsc, ts, dst_min);
         } else {
             warning(0, "DLR[%s]: Storage don't have remove operation defined", dlr_type());
         }
@@ -438,6 +468,7 @@ Msg *dlr_find(const Octstr *smsc, const Octstr *ts, const Octstr *dst, int typ)
 
     /* destroy struct dlr_entry */
     dlr_entry_destroy(dlr);
+    octstr_destroy(dst_min);
 
     return msg;
 }
@@ -477,6 +508,7 @@ Msg* create_dlr_from_msg(const Octstr *smsc, const Msg *msg, const Octstr *reply
     dlrmsg->sms.boxc_id = octstr_duplicate(msg->sms.boxc_id);
     dlrmsg->sms.foreign_id = octstr_duplicate(msg->sms.foreign_id);
     time(&dlrmsg->sms.time);
+    dlrmsg->sms.meta_data = octstr_duplicate(msg->sms.meta_data);
 
     debug("dlr.dlr", 0,"SMSC[%s]: DLR = %s",
                 (smsc ? octstr_get_cstr(smsc) : "UNKNOWN"),
