@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -79,18 +79,23 @@ enum msg_type {
 typedef struct {
 	enum msg_type type;
 
-	#define INTEGER(name) long name
-	#define OCTSTR(name) Octstr *name
-	#define UUID(name) uuid_t name
-	#define VOID(name) void *name
+	#define INTEGER(name) long name;
+	#define OCTSTR(name) Octstr *name;
+	#define UUID(name) uuid_t name;
+	#define VOID(name) void *name;
 	#define MSG(type, stmt) struct type stmt type;
 	#include "msg-decl.h"
 } Msg;
 
 struct split_parts {
+    /* original large message */
     Msg *orig;
+    /* how many parts still not sent */
     Counter *parts_left;
+    /* status of splitted message parts */
     long status;
+    /* pointer to SMSCConn */
+    void *smsc_conn;
 };
 
 /* enums for Msg fields */
