@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -136,25 +136,25 @@ static ota_2table_t ota_elements[] = {
  */
 
 static ota_2table_t ota_syncsettings_elements[] = {
-    { "Version", 0x58 },
-    { "HostAddr", 0x50 },
-    { "Port", 0x52 },
-    { "RemoteDB", 0x54 },
-    { "CTType", 0x4E },
-    { "CTVer", 0x4F },
-    { "URI", 0x56 },
-    { "Name", 0x51 },
+    { "Addr", 0x45 },
+    { "AddrType", 0x46 },
     { "Auth", 0x47 },
     { "AuthLevel", 0x48 },
     { "AuthScheme", 0x49 },
-    { "Username", 0x57 },
-    { "Cred", 0x4D },
-    { "ConRef", 0x4B },
-    { "ConType", 0x4E },
     { "Bearer", 0x4A },
-    { "AddrType", 0x46 },
-    { "Addr", 0x45 },
-    { "RefID", 0x53 }
+    { "ConRef", 0x4B },
+    { "ConType", 0x4C },
+    { "Cred", 0x4D },
+    { "CTType", 0x4E },
+    { "CTVer", 0x4F },
+    { "HostAddr", 0x50 },
+    { "Name", 0x51 },
+    { "Port", 0x52 },
+    { "RefID", 0x53 },
+    { "RemoteDB", 0x54 },
+    { "URI", 0x56 },
+    { "Username", 0x57 },
+    { "Version", 0x58 }
 };
 
 #define NUMBER_OF_SYNCSETTINGS_ELEMENTS sizeof(ota_syncsettings_elements)/sizeof(ota_syncsettings_elements[0])
@@ -234,9 +234,8 @@ static ota_3table_t ota_attributes[] = {
  */
 
 static ota_3table_t oma_ota_attributes[] = {
-    { "VERSION", "INLINE", 0x45 },  
     { "VERSION", "1.0", 0x46 },
-    { "TYPE", "INLINE", 0x50 },
+    { "VERSION", "INLINE", 0x45 },
     { "TYPE", "PXLOGICAL", 0x51 },
     { "TYPE", "PXPHYSICAL", 0x52 },
     { "TYPE", "PORT", 0x53 },
@@ -259,7 +258,7 @@ static ota_3table_t oma_ota_attributes[] = {
     { "TYPE", "EAP", 0x5C, 1 },
     { "TYPE", "CERT", 0x5D, 1 },
     { "TYPE", "WEPKEY", 0x5E, 1 },
-    { "NAME", "INLINE", 0x5 },
+    { "TYPE", "INLINE", 0x50 },
     { "NAME", "NAME", 0x7 },
     { "NAME", "NAP-ADDRESS", 0x8 },
     { "NAME", "NAP-ADDRTYPE", 0x9 },
@@ -378,7 +377,7 @@ static ota_3table_t oma_ota_attributes[] = {
     { "NAME", "EDIT-SET", 0x71, 1 },
     { "NAME", "VIEW-SET", 0x72, 1 },
     { "NAME", "FORW-SET", 0x73, 1 },
-    { "VALUE", "INLINE", 0x6 },     
+    { "NAME", "INLINE", 0x5 },
     { "VALUE", "IPV4", 0x85 },
     { "VALUE", "IPV6", 0x86 },
     { "VALUE", "E164", 0x87 },
@@ -445,6 +444,7 @@ static ota_3table_t oma_ota_attributes[] = {
     { "VALUE", "DIGEST", 0x93, 1 },
     { "VALUE", "AAA", 0xE0 },
     { "VALUE", "HA", 0xE1 },
+    { "VALUE", "INLINE", 0x6 },
 };
 
 #define OMA_VALUE_TAG 0x06
