@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -60,108 +60,17 @@
  * Lars Wirzenius
  */
 
-#include <sys/utsname.h>
+#include <errno.h>
+#include <sys/unistd.h>
 #include <libxml/xmlversion.h>
 
 #include "gwlib/gwlib.h"
 #include "shared.h"
 
-#if defined(HAVE_LIBSSL) || defined(HAVE_WTLS_OPENSSL) 
-#include <openssl/opensslv.h>
-#endif
-#ifdef HAVE_MYSQL 
-#include <mysql_version.h>
-#include <mysql.h>
-#endif
-#ifdef HAVE_SQLITE 
-#include <sqlite.h>
-#endif
-#ifdef HAVE_SQLITE3 
-#include <sqlite3.h>
-#endif
-#ifdef HAVE_ORACLE 
-#include <oci.h>
-#endif
-
 
 volatile enum program_status program_status = starting_up;
 
 
-void report_versions(const char *boxname)
-{
-    Octstr *os;
-    
-    os = version_report_string(boxname);
-    debug("gwlib.gwlib", 0, "%s", octstr_get_cstr(os));
-    octstr_destroy(os);
-}
-
-
-Octstr *version_report_string(const char *boxname)
-{
-    struct utsname u;
-
-    uname(&u);
-    return octstr_format(GW_NAME " %s version `%s'.\nBuild `%s', compiler `%s'.\n"
-    	    	    	 "System %s, release %s, version %s, machine %s.\n"
-			 "Hostname %s, IP %s.\n"
-			 "Libxml version %s.\n"
-#ifdef HAVE_LIBSSL
-             "Using "
-#ifdef HAVE_WTLS_OPENSSL
-             "WTLS library "
-#endif
-             "%s.\n"
-#endif
-#ifdef HAVE_MYSQL
-             "Compiled with MySQL %s, using MySQL %s.\n"
-#endif
-#ifdef HAVE_SDB
-             "Using LibSDB %s.\n"
-#endif
-#if defined(HAVE_SQLITE) || defined(HAVE_SQLITE3)
-             "Using SQLite %s.\n"
-#endif
-#ifdef HAVE_ORACLE
-#if defined(OCI_MAJOR_VERSION) && defined(OCI_MINOR_VERSION)
-             "Using Oracle OCI %d.%d.\n"
-#else
-             "Using Oracle OCI.\n"
-#endif
-#endif
-             "Using %s malloc.\n",
-			 boxname, GW_VERSION,
-#ifdef __GNUC__ 
-             (__DATE__ " " __TIME__) ,
-             __VERSION__,
-#else 
-             "unknown" , "unknown",
-#endif 
-			 u.sysname, u.release, u.version, u.machine,
-			 octstr_get_cstr(get_official_name()),
-			 octstr_get_cstr(get_official_ip()),
-			 LIBXML_DOTTED_VERSION,
-#ifdef HAVE_LIBSSL
-             OPENSSL_VERSION_TEXT,
-#endif
-#ifdef HAVE_MYSQL
-             MYSQL_SERVER_VERSION, mysql_get_client_info(),
-#endif
-#ifdef HAVE_SDB
-             LIBSDB_VERSION,
-#endif
-#if defined(HAVE_SQLITE) || defined(HAVE_SQLITE3)
-             SQLITE_VERSION,
-#endif
-#ifdef HAVE_ORACLE
-#if defined(OCI_MAJOR_VERSION) && defined(OCI_MINOR_VERSION)
-             OCI_MAJOR_VERSION, OCI_MINOR_VERSION,
-#endif
-#endif
-             octstr_get_cstr(gwmem_type()));
-}
-
-
 /***********************************************************************
  * Communication with the bearerbox.
  */
@@ -367,3 +276,21 @@ error:
     return NULL;
 }
 
+int restart_box(char **argv)
+{
+    int rc;
+
+    if (!(rc = fork())) {
+        /*
+         * Sleep a while in order to get father
+         * process time to cleanup things
+         */
+        gwthread_sleep(1.0);
+        if (execvp(argv[0],argv) == -1)
+            error(errno, "Unable to start new process.");
+    } else if (rc == -1)
+        error(errno, "Could not restart, exiting...");
+
+    return rc == -1 ? -1 : 0;
+}
+
