@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -58,7 +58,7 @@
  * smpp_pdu.c - parse and generate SMPP PDUs
  *
  * Lars Wirzenius
- * Alexander Malysh <a.malysh@centrium.de>:
+ * Alexander Malysh <amalysh@kannel.org>:
  *     Extended optional parameters implementation.
  */
 
@@ -69,6 +69,209 @@
 #define MIN_SMPP_PDU_LEN    (4*4)
 /* old value was (1024). We need more because message_payload can be up to 64K octets*/
 #define MAX_SMPP_PDU_LEN    (7424)
+/* we use ; in the middle because ; is split char in smsc-id and can't be in the smsc-id */
+#define DEFAULT_SMSC_ID "def;ault"
+
+struct smpp_tlv {
+    Octstr *name;
+    long tag;
+    long length;
+    enum { SMPP_TLV_OCTETS = 0, SMPP_TLV_NULTERMINATED = 1, SMPP_TLV_INTEGER = 2 } type;
+};
+
+/* Dict(smsc_id, Dict(tag, tlv)) */
+static Dict *tlvs_by_tag;
+/* Dict(smsc_id, Dict(tag_name, tlv)) */
+static Dict *tlvs_by_name;
+static List *tlvs;
+static int initialized;
+
+
+static void smpp_tlv_destroy(struct smpp_tlv *tlv)
+{
+    if (tlv == NULL)
+        return;
+    octstr_destroy(tlv->name);
+    gw_free(tlv);
+}
+
+static struct smpp_tlv *smpp_tlv_get_by_name(Octstr *smsc_id, Octstr *name)
+{
+    struct smpp_tlv *res = NULL;
+    Dict *tmp_dict;
+
+    if (tlvs_by_name == NULL || name == NULL)
+        return NULL;
+
+    if (smsc_id != NULL) {
+        tmp_dict = dict_get(tlvs_by_name, smsc_id);
+        if (tmp_dict != NULL)
+            res = dict_get(tmp_dict, name);
+    }
+    if (res == NULL) {
+        /* try default smsc_id */
+        smsc_id = octstr_imm(DEFAULT_SMSC_ID);
+        tmp_dict = dict_get(tlvs_by_name, smsc_id);
+        if (tmp_dict != NULL)
+            res = dict_get(tmp_dict, name);
+    }
+    return res;
+}
+
+static struct smpp_tlv *smpp_tlv_get_by_tag(Octstr *smsc_id, long tag)
+{
+    struct smpp_tlv *res = NULL;
+    Dict *tmp_dict;
+    Octstr *tmp;
+
+    if (tlvs_by_tag == NULL)
+        return NULL;
+
+    tmp = octstr_format("%ld", tag);
+
+    if (smsc_id != NULL) {
+        tmp_dict = dict_get(tlvs_by_tag, smsc_id);
+        if (tmp_dict != NULL)
+            res = dict_get(tmp_dict, tmp);
+    }
+    if (res == NULL) {
+        /* try default smsc_id */
+        smsc_id = octstr_imm(DEFAULT_SMSC_ID);
+        tmp_dict = dict_get(tlvs_by_tag, smsc_id);
+        if (tmp_dict != NULL)
+            res = dict_get(tmp_dict, tmp);
+    }
+
+    octstr_destroy(tmp);
+
+    return res;
+}
+
+int smpp_pdu_init(Cfg *cfg)
+{
+    CfgGroup *grp;
+    List *l;
+
+    if (initialized)
+        return 0;
+
+    l = cfg_get_multi_group(cfg, octstr_imm("smpp-tlv"));
+    tlvs = gwlist_create();
+    tlvs_by_tag = dict_create(1024, (void(*)(void*))dict_destroy);
+    tlvs_by_name = dict_create(1024, (void(*)(void*))dict_destroy);
+    while (l != NULL && (grp = gwlist_extract_first(l)) != NULL) {
+        struct smpp_tlv *tlv;
+        Octstr *tmp, *smsc_id;
+        List *l2;
+
+        tlv = gw_malloc(sizeof(*tlv));
+        if ((tlv->name = cfg_get(grp, octstr_imm("name"))) == NULL) {
+            error(0, "SMPP: Unable to get name for smpp-tlv.");
+            smpp_tlv_destroy(tlv);
+            goto failed;
+        }
+        if (cfg_get_integer(&tlv->tag, grp, octstr_imm("tag")) == -1) {
+            error(0, "SMPP: Unable to get tag for smpp-tlv.");
+            smpp_tlv_destroy(tlv);
+            goto failed;
+        }
+        if (cfg_get_integer(&tlv->length, grp, octstr_imm("length")) == -1) {
+            error(0, "SMPP: Unable to get length for smpp-tlv.");
+            smpp_tlv_destroy(tlv);
+            goto failed;
+        }
+        if ((tmp = cfg_get(grp, octstr_imm("type"))) == NULL) {
+            error(0, "SMPP: Unable to get type for smpp-tlv.");
+            smpp_tlv_destroy(tlv);
+            goto failed;
+        }
+        if (octstr_str_case_compare(tmp, "octetstring") == 0)
+            tlv->type = SMPP_TLV_OCTETS;
+        else if (octstr_str_case_compare(tmp, "nulterminated") == 0)
+            tlv->type = SMPP_TLV_NULTERMINATED;
+        else if (octstr_str_case_compare(tmp, "integer") == 0)
+            tlv->type = SMPP_TLV_INTEGER;
+        else {
+            error(0, "SMPP: Unknown type for smpp-tlv: `%s'", octstr_get_cstr(tmp));
+            octstr_destroy(tmp);
+            smpp_tlv_destroy(tlv);
+            goto failed;
+        }
+        octstr_destroy(tmp);
+
+        /* put to all TLVs */
+        gwlist_produce(tlvs, tlv);
+
+        smsc_id = cfg_get(grp, octstr_imm("smsc-id"));
+        if (smsc_id != NULL) {
+            l2 = octstr_split(smsc_id, octstr_imm(";"));
+            octstr_destroy(smsc_id);
+        } else {
+            l2 = gwlist_create();
+            gwlist_produce(l2, octstr_create(DEFAULT_SMSC_ID));
+        }
+        while(l2 != NULL && (smsc_id = gwlist_extract_first(l2)) != NULL) {
+            Dict *tmp_dict;
+
+            debug("sms.smpp", 0, "adding smpp-tlv for smsc-id=%s", octstr_get_cstr(smsc_id));
+
+            tmp_dict = dict_get(tlvs_by_name, smsc_id);
+            if (tmp_dict == NULL) {
+                tmp_dict = dict_create(1024, NULL);
+                dict_put(tlvs_by_name, smsc_id, tmp_dict);
+            }
+            /* put into dict */
+            if (!dict_put_once(tmp_dict, tlv->name, tlv)) {
+                error(0, "SMPP: Double TLV name %s found.", octstr_get_cstr(tlv->name));
+                octstr_destroy(smsc_id);
+                goto failed;
+            }
+
+            tmp_dict = dict_get(tlvs_by_tag, smsc_id);
+            if (tmp_dict == NULL) {
+                tmp_dict = dict_create(1024, NULL);
+                dict_put(tlvs_by_tag, smsc_id, tmp_dict);
+            }
+            tmp = octstr_format("%ld", tlv->tag);
+            if (!dict_put_once(tmp_dict, tmp, tlv)) {
+                error(0, "SMPP: Double TLV tag %s found.", octstr_get_cstr(tmp));
+                gwlist_destroy(l2, octstr_destroy_item);
+                octstr_destroy(tmp);
+                octstr_destroy(smsc_id);
+                goto failed;
+            }
+            octstr_destroy(tmp);
+            octstr_destroy(smsc_id);
+        }
+        gwlist_destroy(l2, octstr_destroy_item);
+    }
+    gwlist_destroy(l, NULL);
+
+    initialized = 1;
+    return 0;
+
+failed:
+    gwlist_destroy(tlvs, (void(*)(void*))smpp_tlv_destroy);
+    dict_destroy(tlvs_by_tag);
+    dict_destroy(tlvs_by_name);
+    return -1;
+}
+
+
+int smpp_pdu_shutdown(void)
+{
+    if (initialized == 0)
+        return 0;
+
+    initialized = 0;
+    gwlist_destroy(tlvs, (void(*)(void*))smpp_tlv_destroy);
+    tlvs = NULL;
+    dict_destroy(tlvs_by_tag);
+    dict_destroy(tlvs_by_name);
+    tlvs_by_tag = tlvs_by_name = NULL;
+
+    return 0;
+}
 
 
 static long decode_integer(Octstr *os, long pos, int octets)
@@ -129,7 +332,7 @@ SMPP_PDU *smpp_pdu_create(unsigned long type, unsigned long seq_no)
     #define TLV_INTEGER(name, octets) p->name = -1;
     #define TLV_NULTERMINATED(name, max_len) p->name = NULL;
     #define TLV_OCTETS(name, min_len, max_len) p->name = NULL;
-    #define OPTIONAL_END
+    #define OPTIONAL_END p->tlv = dict_create(1024, octstr_destroy_item);
     #define INTEGER(name, octets) p->name = 0;
     #define NULTERMINATED(name, max_octets) p->name = NULL;
     #define OCTETS(name, field_giving_octetst) p->name = NULL;
@@ -161,7 +364,7 @@ void smpp_pdu_destroy(SMPP_PDU *pdu)
     #define TLV_INTEGER(name, octets) p->name = -1;
     #define TLV_NULTERMINATED(name, max_octets) octstr_destroy(p->name);
     #define TLV_OCTETS(name, min_len, max_len) octstr_destroy(p->name);
-    #define OPTIONAL_END
+    #define OPTIONAL_END dict_destroy(p->tlv);
     #define INTEGER(name, octets) p->name = 0; /* Make sure "p" is used */
     #define NULTERMINATED(name, max_octets) octstr_destroy(p->name);
     #define OCTETS(name, field_giving_octets) octstr_destroy(p->name);
@@ -175,7 +378,7 @@ void smpp_pdu_destroy(SMPP_PDU *pdu)
 }
 
 
-Octstr *smpp_pdu_pack(SMPP_PDU *pdu)
+Octstr *smpp_pdu_pack(Octstr *smsc_id, SMPP_PDU *pdu)
 {
     Octstr *os;
     Octstr *temp;
@@ -210,7 +413,7 @@ Octstr *smpp_pdu_pack(SMPP_PDU *pdu)
         append_encoded_integer(os, octets, 2);
     #define OPTIONAL_BEGIN
     #define TLV_INTEGER(name, octets) \
-        if (p->name != -1) { \
+        if (p->name >= 0) { \
             TL(name, octets); \
             INTEGER(name, octets) \
         }
@@ -230,7 +433,55 @@ Octstr *smpp_pdu_pack(SMPP_PDU *pdu)
                 octstr_append(os, p->name); \
             } \
         }
-    #define OPTIONAL_END
+    #define OPTIONAL_END \
+        if (p->tlv != NULL) { \
+            Octstr *key; \
+            List *keys; \
+            struct smpp_tlv *tlv; \
+            keys = dict_keys(p->tlv); \
+            while(keys != NULL && (key = gwlist_extract_first(keys)) != NULL) { \
+                tlv = smpp_tlv_get_by_name(smsc_id, key); \
+                if (tlv == NULL) { \
+                    error(0, "SMPP: Unknown TLV `%s', don't send.", octstr_get_cstr(key)); \
+                    octstr_destroy(key); \
+                    continue; \
+                } \
+                switch(tlv->type) { \
+                case SMPP_TLV_INTEGER: { \
+                    long val = atol(octstr_get_cstr(dict_get(p->tlv, key))); \
+                    append_encoded_integer(os, tlv->tag, 2); \
+                    append_encoded_integer(os, tlv->length, 2); \
+                    append_encoded_integer(os, val, tlv->length); \
+                    break; \
+                } \
+                case SMPP_TLV_OCTETS: \
+                case SMPP_TLV_NULTERMINATED: { \
+                    Octstr *val = dict_get(p->tlv, key); \
+                    unsigned long len = octstr_len(val); \
+                    if (len > tlv->length) { \
+                        error(0, "SMPP: Optional field (%s) with invalid length (%ld) (should be %ld) dropped.", \
+                              octstr_get_cstr(key), len, tlv->length);\
+                        octstr_destroy(key); \
+                        continue; \
+                    } \
+                    append_encoded_integer(os, tlv->tag, 2); \
+                    if (tlv->type == SMPP_TLV_NULTERMINATED) \
+                        append_encoded_integer(os, len + 1, 2); \
+                    else \
+                        append_encoded_integer(os, len, 2); \
+                    octstr_append(os, val); \
+                    if (tlv->type == SMPP_TLV_NULTERMINATED) \
+                        octstr_append_char(os, '\0'); \
+                    break; \
+                } \
+                default: \
+                    panic(0, "SMPP: Internal error, unknown configured TLV type %d.", tlv->type); \
+                    break; \
+                } \
+                octstr_destroy(key); \
+            } \
+            gwlist_destroy(keys, octstr_destroy_item); \
+        }
     #define INTEGER(name, octets) \
         append_encoded_integer(os, p->name, octets);
     #define NULTERMINATED(name, max_octets) \
@@ -252,7 +503,8 @@ Octstr *smpp_pdu_pack(SMPP_PDU *pdu)
         case id: { struct name *p = &pdu->u.name; fields } break;
     #include "smpp_pdu.def"
     default:
-        error(0, "Unknown SMPP_PDU type, internal error while packing.");
+        error(0, "Unknown SMPP_PDU type 0x%08lx, internal error while packing.", pdu->type);
+        break;
     }
 
     temp = octstr_create("");
@@ -264,7 +516,7 @@ Octstr *smpp_pdu_pack(SMPP_PDU *pdu)
 }
 
 
-SMPP_PDU *smpp_pdu_unpack(Octstr *data_without_len)
+SMPP_PDU *smpp_pdu_unpack(Octstr *smsc_id, Octstr *data_without_len)
 {
     SMPP_PDU *pdu;
     unsigned long type;
@@ -293,52 +545,98 @@ SMPP_PDU *smpp_pdu_unpack(Octstr *data_without_len)
     #define OPTIONAL_BEGIN  \
         {   /* Read optional parameters */  \
             while (pos + 4 <= len) { \
+                struct smpp_tlv *tlv; \
                 unsigned long opt_tag, opt_len; \
                 opt_tag = decode_integer(data_without_len, pos, 2); pos += 2; \
-                debug("sms.smpp", 0, "Optional parameter tag (0x%04lx)", opt_tag);   \
+                debug("sms.smpp", 0, "Optional parameter tag (0x%04lx)", opt_tag);  \
                 opt_len = decode_integer(data_without_len, pos, 2); pos += 2;  \
-                debug("sms.smpp", 0, "Optional parameter length read as %ld", opt_len);
-    #define TLV_INTEGER(name, octets) \
-                if (SMPP_##name == opt_tag) { \
+                debug("sms.smpp", 0, "Optional parameter length read as %ld", opt_len); \
+                /* check configured TLVs */ \
+                tlv = smpp_tlv_get_by_tag(smsc_id, opt_tag); \
+                if (tlv != NULL) debug("sms.smpp", 0, "Found configured optional parameter `%s'", octstr_get_cstr(tlv->name));
+    #define TLV_INTEGER(mname, octets) \
+                if (SMPP_##mname == opt_tag) { \
                     /* check length */ \
                     if (opt_len > octets) { \
-                        error(0, "SMPP: Optional field (%s) with invalid length (%ld) dropped.", #name, opt_len); \
+                        error(0, "SMPP: Optional field (%s) with invalid length (%ld) dropped.", #mname, opt_len); \
                         pos += opt_len; \
                         continue; \
                     } \
-                    INTEGER(name, opt_len); \
+                    INTEGER(mname, opt_len); \
+                    if (tlv != NULL) dict_put(p->tlv, tlv->name, octstr_format("%ld", p->mname)); \
                 } else
-    #define TLV_NULTERMINATED(name, max_len) \
-                if (SMPP_##name == opt_tag) { \
+    #define TLV_NULTERMINATED(mname, max_len) \
+                if (SMPP_##mname == opt_tag) { \
                     /* check length */ \
                     if (opt_len > max_len || pos+opt_len > len) { \
-                        error(0, "SMPP: Optional field (%s) with invalid length (%ld) dropped.", #name, opt_len);  \
+                        error(0, "SMPP: Optional field (%s) with invalid length (%ld) dropped.", #mname, opt_len);  \
                         pos += opt_len; \
                         continue; \
                     } \
-                    copy_until_nul(#name, data_without_len, &pos, opt_len, &p->name); \
+                    copy_until_nul(#mname, data_without_len, &pos, opt_len, &p->mname); \
+                    if (tlv != NULL) dict_put(p->tlv, tlv->name, octstr_duplicate(p->mname)); \
                 } else
-    #define TLV_OCTETS(name, min_len, max_len) \
-                if (SMPP_##name == opt_tag) { \
+    #define TLV_OCTETS(mname, min_len, max_len) \
+                if (SMPP_##mname == opt_tag) { \
                     /* check length */ \
                     if (opt_len < min_len || opt_len > max_len || pos + opt_len > len) { \
                         error(0, "SMPP: Optional field (%s) with invalid length (%ld) (should be %d - %d) dropped.", \
-                            #name, opt_len, min_len, max_len);  \
+                            #mname, opt_len, min_len, max_len);  \
                         pos += opt_len; \
                         continue; \
                     } \
-                    p->name = octstr_copy(data_without_len, pos, opt_len); \
+                    p->mname = octstr_copy(data_without_len, pos, opt_len); \
                     pos += opt_len; \
+                    if (tlv != NULL) dict_put(p->tlv, tlv->name, octstr_duplicate(p->mname)); \
                 } else
     #define OPTIONAL_END \
                 { \
-                    Octstr *val = octstr_copy(data_without_len, pos, opt_len); \
-                    if (val) octstr_binary_to_hex(val, 0); \
-                    else val = octstr_create(""); \
-                    warning(0, "SMPP: Unknown TLV(0x%04lx,0x%04lx,%s) for PDU type (%s) received!", \
+                    Octstr *val = NULL; \
+                    if (tlv != NULL) { \
+                        /* found configured tlv */ \
+                        /* check length */ \
+                        if (opt_len > tlv->length) { \
+                            error(0, "SMPP: Optional field (%s) with invalid length (%ld) (should be %ld) dropped.", \
+                                  octstr_get_cstr(tlv->name), opt_len, tlv->length); \
+                            pos += opt_len; \
+                            continue; \
+                        } \
+                        switch (tlv->type) { \
+                        case SMPP_TLV_INTEGER: { \
+                            long val_i; \
+                            if ((val_i = decode_integer(data_without_len, pos, opt_len)) == -1) \
+                                goto err; \
+                            val = octstr_format("%ld", val_i); \
+                            dict_put(p->tlv, tlv->name, val); \
+                            pos += opt_len; \
+                            break; \
+                        } \
+                        case SMPP_TLV_OCTETS: { \
+                            val = octstr_copy(data_without_len, pos, opt_len); \
+                            dict_put(p->tlv, tlv->name, val); \
+                            pos += opt_len; \
+                            break; \
+                        } \
+                        case SMPP_TLV_NULTERMINATED: { \
+                            if (copy_until_nul(octstr_get_cstr(tlv->name), data_without_len, &pos, opt_len, &val) == 0) \
+                                dict_put(p->tlv, tlv->name, val); \
+                            break; \
+                        } \
+                        default: \
+                            panic(0, "SMPP: Internal error, unknown configured TLV type %d.", tlv->type); \
+                            break; \
+                        } \
+                    }  else { \
+                        val = octstr_copy(data_without_len, pos, opt_len); \
+                        if (val) \
+                            octstr_binary_to_hex(val, 0); \
+                        else \
+                            val = octstr_create(""); \
+                        warning(0, "SMPP: Unknown TLV(0x%04lx,0x%04lx,%s) for PDU type (%s) received!", \
                             opt_tag, opt_len, octstr_get_cstr(val), pdu->type_name); \
-                    pos += opt_len; \
-                    octstr_destroy(val); \
+                        octstr_destroy(val); \
+                        pos += opt_len; \
+                    } \
                 } \
             } \
         }
@@ -364,7 +662,8 @@ SMPP_PDU *smpp_pdu_unpack(Octstr *data_without_len)
     	case id: { struct name *p = &pdu->u.name; fields } break;
     #include "smpp_pdu.def"
     default:
-    	error(0, "Unknown SMPP_PDU type, internal error while unpacking.");
+    	error(0, "Unknown SMPP_PDU type 0x%08lx, internal error while unpacking.", type);
+    	break;
     }
 
     return pdu;
@@ -376,7 +675,7 @@ err:
 }
 
 
-void smpp_pdu_dump(SMPP_PDU *pdu)
+void smpp_pdu_dump(Octstr *smsc_id, SMPP_PDU *pdu)
 {
     debug("sms.smpp", 0, "SMPP PDU %p dump:", (void *) pdu);
     debug("sms.smpp", 0, "  type_name: %s", pdu->type_name);
@@ -394,7 +693,21 @@ void smpp_pdu_dump(SMPP_PDU *pdu)
         if (p->name != NULL) { \
             OCTETS(name, max_len) \
         }
-    #define OPTIONAL_END
+    #define OPTIONAL_END \
+        if (p->tlv != NULL) { \
+            List *keys; \
+            Octstr *key; \
+            struct smpp_tlv *tlv; \
+            keys = dict_keys(p->tlv); \
+            while(keys != NULL && (key = gwlist_extract_first(keys)) != NULL) { \
+                tlv = smpp_tlv_get_by_name(smsc_id, key); \
+                if (tlv != NULL) { \
+                    octstr_dump_short(dict_get(p->tlv, key), 2, octstr_get_cstr(key)); \
+                } \
+                octstr_destroy(key); \
+            } \
+            gwlist_destroy(keys, octstr_destroy_item); \
+        }
     #define INTEGER(name, octets) \
         debug("sms.smpp", 0, "  %s: %lu = 0x%08lx", #name, p->name, p->name);
     #define NULTERMINATED(name, max_octets) \
