@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -348,7 +348,7 @@ error:
 int smscenter_read_into_buffer(SMSCenter *smsc)
 {
     char *p;
-    int ret, result;
+    int ret;
     fd_set read_fd;
     struct timeval tv, tvinit;
     size_t bytes_read;
@@ -357,7 +357,6 @@ int smscenter_read_into_buffer(SMSCenter *smsc)
     tvinit.tv_usec = 1000;
 
     bytes_read = 0;
-    result = 0;
     for (;;) {
         FD_ZERO(&read_fd);
         FD_SET(smsc->socket, &read_fd);
