@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -502,7 +502,7 @@ static struct packet *packet_encode_message(Msg *msg, SMSCConn *conn)
     struct packet *packet;
     PrivData *pdata = conn->data;
     int DCS;
-    int setvalidity = 0;
+    int setvalidity = SMS_PARAM_UNDEFINED;
     int so = 0;
     int udhlen7, udhlen8;
     int msglen7, msglen8;
@@ -577,8 +577,11 @@ static struct packet *packet_encode_message(Msg *msg, SMSCConn *conn)
      * Validity-Period (TP-VP)
      * see GSM 03.40 section 9.2.3.12
      */
-    if ((setvalidity = msg->sms.validity) != SMS_PARAM_UNDEFINED ||
-        (setvalidity = pdata->validityperiod) != SMS_PARAM_UNDEFINED) {
+    if (msg->sms.validity != SMS_PARAM_UNDEFINED)
+    	setvalidity = (msg->sms.validity - time(NULL)) / 60;
+    else if (setvalidity != SMS_PARAM_UNDEFINED)
+    	setvalidity = pdata->validityperiod;
+    if (setvalidity != SMS_PARAM_UNDEFINED) {
         /* Validity period type 0=none, 1=absolute, 2=relative */
         octstr_append_char(packet->data, 2);
 
@@ -894,9 +897,7 @@ static Msg *oisd_accept_message(struct packet *request, SMSCConn *conn)
 
     if ((!(msg->sms.msgdata) || octstr_len(msg->sms.msgdata) == 0)
         && (!(msg->sms.udhdata) || octstr_len(msg->sms.udhdata) == 0)) {
-        info(0, "OISD[%s]: Got empty SMS, ignoring.",
-              octstr_get_cstr(conn->id));
-        goto error;
+        msg->sms.msgdata = octstr_create("");
     }
 
     return msg;
@@ -1171,7 +1172,7 @@ static int oisd_submit_msg(SMSCConn *conn, Msg *msg)
 
     ret = oisd_request(packet, conn, &ts);
     if((ret == 0) && (ts) && DLR_IS_SUCCESS_OR_FAIL(msg->sms.dlr_mask) && !pdata->no_dlr) {
-        dlr_add(conn->name, ts, msg);
+        dlr_add(conn->name, ts, msg, 0);
     }
     octstr_destroy(ts);
     packet_destroy(packet);
@@ -1308,7 +1309,7 @@ static Msg *oisd_accept_delivery_report_message(struct packet *request,
     }
 
     if (code)
-        msg = dlr_find(conn->name, timestamp, destination, code);
+        msg = dlr_find(conn->name, timestamp, destination, code, 0);
 
     octstr_destroy(destination);
     octstr_destroy(timestamp);
@@ -1523,7 +1524,7 @@ int smsc_oisd_create(SMSCConn *conn, CfgGroup *grp)
     if (cfg_get_integer(&(pdata->keepalive), grp, octstr_imm("keepalive")) == -1)
         pdata->keepalive = 0;
     if (cfg_get_integer(&(pdata->validityperiod), grp, octstr_imm("validityperiod")) == -1)
-        pdata->validityperiod = 0;
+        pdata->validityperiod = SMS_PARAM_UNDEFINED;
 
     cfg_get_bool(&pdata->no_dlr, grp, octstr_imm("no-dlr"));
 
