@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -911,7 +911,6 @@ static void send_messages(SMASI *smasi, Connection *conn,
 static void smasi_thread(void *arg) 
 {
     long pending_submits;
-    long len;
     SMASI_PDU *pdu;
     SMASI *smasi;
     int logoff_already_sent = 0;
@@ -940,7 +939,6 @@ static void smasi_thread(void *arg)
 
         last_enquire_sent = date_universal_now(); 
         pending_submits = -1;
-        len = 0;
 
         for (;;) {
             timeout = last_enquire_sent + smasi->enquire_link_interval
