@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -170,10 +170,37 @@ SMSCConn *smscconn_create(CfgGroup *grp, int start_as_stopped)
     conn->is_stopped = start_as_stopped;
 
     conn->received = counter_create();
+    conn->received_dlr = counter_create();
     conn->sent = counter_create();
+    conn->sent_dlr = counter_create();
     conn->failed = counter_create();
     conn->flow_mutex = mutex_create();
 
+    conn->outgoing_sms_load = load_create();
+    /* add 60,300,-1 entries */
+    load_add_interval(conn->outgoing_sms_load, 60);
+    load_add_interval(conn->outgoing_sms_load, 300);
+    load_add_interval(conn->outgoing_sms_load, -1);
+
+    conn->incoming_sms_load = load_create();
+    /* add 60,300,-1 entries */
+    load_add_interval(conn->incoming_sms_load, 60);
+    load_add_interval(conn->incoming_sms_load, 300);
+    load_add_interval(conn->incoming_sms_load, -1);
+
+    conn->incoming_dlr_load = load_create();
+    /* add 60,300,-1 entries to dlr */
+    load_add_interval(conn->incoming_dlr_load, 60);
+    load_add_interval(conn->incoming_dlr_load, 300);
+    load_add_interval(conn->incoming_dlr_load, -1);
+
+    conn->outgoing_dlr_load = load_create();
+    /* add 60,300,-1 entries to dlr */
+    load_add_interval(conn->outgoing_dlr_load, 60);
+    load_add_interval(conn->outgoing_dlr_load, 300);
+    load_add_interval(conn->outgoing_dlr_load, -1);
+
+
 #define GET_OPTIONAL_VAL(x, n) x = cfg_get(grp, octstr_imm(n))
 #define SPLIT_OPTIONAL_VAL(x, n) \
         do { \
@@ -222,6 +249,10 @@ SMSCConn *smscconn_create(CfgGroup *grp, int start_as_stopped)
         octstr_destroy(tmp);
         info(0, "Set throughput to %.3f for smsc id <%s>", conn->throughput, octstr_get_cstr(conn->id));
     }
+    /* Sets the admin_id. Equals to connection id if empty */
+    GET_OPTIONAL_VAL(conn->admin_id, "smsc-admin-id");
+    if (conn->admin_id == NULL)
+        conn->admin_id = octstr_duplicate(conn->id);
 
     /* configure the internal rerouting rules for this smsc id */
     init_reroute(conn, grp);
@@ -232,6 +263,9 @@ SMSCConn *smscconn_create(CfgGroup *grp, int start_as_stopped)
     if (cfg_get_integer(&conn->max_sms_octets, grp, octstr_imm("max-sms-octets")) == -1)
         conn->max_sms_octets = MAX_SMS_OCTETS;
 
+    if (cfg_get_bool(&conn->dead_start, grp, octstr_imm("dead-start")) == -1)
+        conn->dead_start = 0;	/* default to connect at start-up time */
+
     /* open a smsc-id specific log-file in exlusive mode */
     if (conn->log_file)
         conn->log_idx = log_open(octstr_get_cstr(conn->log_file), 
@@ -276,6 +310,12 @@ SMSCConn *smscconn_create(CfgGroup *grp, int start_as_stopped)
         ret = smsc_smasi_create(conn, grp);
     else if (octstr_compare(smsc_type, octstr_imm("oisd")) == 0)
         ret = smsc_oisd_create(conn, grp);
+    else if (octstr_compare(smsc_type, octstr_imm("loopback")) == 0)
+        ret = smsc_loopback_create(conn, grp);
+#ifdef HAVE_GSOAP
+    else if (octstr_compare(smsc_type, octstr_imm("parlayx")) == 0)
+    	ret = smsc_soap_parlayx_create(conn, grp);
+#endif
     else
         ret = smsc_wrapper_create(conn, grp);
 
@@ -328,11 +368,19 @@ int smscconn_destroy(SMSCConn *conn)
     mutex_lock(conn->flow_mutex);
 
     counter_destroy(conn->received);
+    counter_destroy(conn->received_dlr);
     counter_destroy(conn->sent);
+    counter_destroy(conn->sent_dlr);
     counter_destroy(conn->failed);
 
+    load_destroy(conn->incoming_sms_load);
+    load_destroy(conn->incoming_dlr_load);
+    load_destroy(conn->outgoing_sms_load);
+    load_destroy(conn->outgoing_dlr_load);
+
     octstr_destroy(conn->name);
     octstr_destroy(conn->id);
+    octstr_destroy(conn->admin_id);
     gwlist_destroy(conn->allowed_smsc_id, octstr_destroy_item);
     gwlist_destroy(conn->denied_smsc_id, octstr_destroy_item);
     gwlist_destroy(conn->preferred_smsc_id, octstr_destroy_item);
@@ -410,6 +458,13 @@ const Octstr *smscconn_id(SMSCConn *conn)
 }
 
 
+const Octstr *smscconn_admin_id(SMSCConn *conn)
+{
+    gw_assert(conn != NULL);
+    return conn->admin_id;
+}
+
+
 int smscconn_usable(SMSCConn *conn, Msg *msg)
 {
     gw_assert(conn != NULL);
@@ -533,6 +588,7 @@ int smscconn_send(SMSCConn *conn, Msg *msg)
         split->parts_left = counter_create();
         split->status = SMSCCONN_SUCCESS;
         counter_set(split->parts_left, parts_len);
+        split->smsc_conn = conn;
         debug("bb.sms.splits", 0, "new split_parts created %p", split);
         for (i = 0; i < parts_len; i++) {
             msg = gwlist_get(parts, i);
@@ -585,6 +641,8 @@ int smscconn_info(SMSCConn *conn, StatusInfo *infotable)
     
     infotable->sent = counter_value(conn->sent);
     infotable->received = counter_value(conn->received);
+    infotable->sent_dlr = counter_value(conn->sent_dlr);
+    infotable->received_dlr = counter_value(conn->received_dlr);
     infotable->failed = counter_value(conn->failed);
 
     if (conn->queued)
