@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -144,6 +144,7 @@
 #include "gwlib/gwlib.h"
 #include "gwlib/regex.h"
 #include "smscconn.h"
+#include "load.h"
 
 struct smscconn {
     /* variables set by appropriate SMSCConn driver */
@@ -159,7 +160,9 @@ struct smscconn {
     /* connection specific counters (created in smscconn.c, updated
      *  by callback functions in bb_smscconn.c, NOT used by specific driver) */
     Counter *received;
+    Counter *received_dlr;
     Counter *sent;
+    Counter *sent_dlr;
     Counter *failed;
 
     /* SMSCConn variables set in smscconn.c */
@@ -168,6 +171,7 @@ struct smscconn {
     Octstr *name;		/* Descriptive name filled from connection info */
     Octstr *id;			/* Abstract name specified in configuration and
 				   used for logging and routing */
+    Octstr *admin_id;
     List *allowed_smsc_id;
     List *denied_smsc_id;
     List *preferred_smsc_id;
@@ -201,9 +205,15 @@ struct smscconn {
     Dict *reroute_by_receiver;  /* reroute receiver numbers to specific smsc-ids */
     Octstr *reroute_to_smsc;    /* define a smsc-id to reroute to */
     int reroute_dlr;            /* should DLR's are rereouted too? */
+    int dead_start;             /* don't connect this SMSC at startup time */
 
     long max_sms_octets; /* max allowed octets for this SMSC */
 
+    Load *outgoing_sms_load;
+    Load *incoming_sms_load;
+    Load *incoming_dlr_load;
+    Load *outgoing_dlr_load;
+
     /* XXX: move rest global data from Smsc here
      */
 
@@ -283,6 +293,14 @@ int smsc_smasi_create(SMSCConn *conn, CfgGroup *cfg);
 /* Responsible file: smsc/smsc_oisd.c */
 int smsc_oisd_create(SMSCConn *conn, CfgGroup *cfg);
 
+/* Responsible file: smsc/smsc_loopback.c */
+int smsc_loopback_create(SMSCConn *conn, CfgGroup *cfg);
+
+#ifdef HAVE_GSOAP
+/* Responsible file: smsc/smsc_soap_parlayx.c */
+int smsc_soap_parlayx_create(SMSCConn *conn, CfgGroup *cfg);
+#endif
+
 /* ADD NEW CREATE FUNCTIONS HERE
  *
  * int smsc_xxx_create(SMSCConn *conn, CfgGroup *cfg);
