@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -286,6 +286,12 @@ Octstr *urltrans_header(URLTranslation *t);
 Octstr *urltrans_footer(URLTranslation *t);
 
 
+/*
+ * Return the alternative charset to be used for the sms-service.
+ */
+Octstr *urltrans_alt_charset(URLTranslation *t);
+
+
 /*
  * return the name, username or password string, or NULL if not set
  * (used only with TRANSTYPE_SENDSMS)
@@ -326,4 +332,8 @@ int urltrans_accept_x_kannel_headers(URLTranslation *t);
 int urltrans_strip_keyword(URLTranslation *t);
 int urltrans_send_sender(URLTranslation *t);
 
+/* Return DLR related values */
+Octstr *urltrans_dlr_url(URLTranslation *t);
+int urltrans_dlr_mask(URLTranslation *t);
+
 #endif
