@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -1462,8 +1462,9 @@ static List *negotiate_capabilities(List *req_caps)
 static void check_application_headers(List **headers, 
                                       List **application_headers)
 {
-    List *inh;
+    List *inh = NULL;
     int i;
+    long len;
     Octstr *appid_name, *coded_octstr;
     char *appid_value, *coded_value;
 
@@ -1479,10 +1480,11 @@ static void check_application_headers(List **headers,
     }
 
     i = 0;
+    len = gwlist_len(inh);
     coded_value = NULL;
     appid_value = NULL;
 
-    while (gwlist_len(inh) > 0) {
+    while (i < len) {
         http_header_get(inh, i, &appid_name, &coded_octstr);
 
         /* Greatest value reserved by WINA is 0xFF00 0000*/
