@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -941,12 +941,9 @@ parsed:
  */
 static int parse_pap_value(Octstr *attr_name, Octstr *attr_value, WAPEvent **e)
 {
-    Octstr *ros;
-
     if (*e != NULL)
         wap_event_dump(*e);
 
-    ros = octstr_imm("erroneous");
     if (octstr_compare(attr_name, octstr_imm("product-name")) == 0) {
         /* 
          * XXX This is a kludge. 
@@ -1104,9 +1101,7 @@ static int parse_push_response_value(Octstr *attr_name, Octstr *attr_value,
                                      WAPEvent **e)
 {
     Octstr *ros;
-    int ret;
 
-    ret = -2;
     ros = octstr_imm("erroneous");
 
     if (octstr_compare(attr_name, octstr_imm("push-id")) == 0) {
@@ -1291,6 +1286,9 @@ static int set_attribute_value(Octstr *element_name, Octstr *attr_value,
 
     ret = -2;
     if (octstr_compare(element_name, octstr_imm("push-message")) == 0) {
+        if (*e == NULL)
+            *e = wap_event_create(Push_Message);
+
         if (octstr_compare(attr_name, 
                           octstr_imm("progress-notes-requested")) == 0)
             (**e).u.Push_Message.progress_notes_requested = 
@@ -1298,7 +1296,10 @@ static int set_attribute_value(Octstr *element_name, Octstr *attr_value,
 
     } else if (octstr_compare(element_name, 
 			     octstr_imm("quality-of-service")) == 0) {
-        if (octstr_compare(attr_name, octstr_imm("priority")) == 0)
+        if (*e == NULL)
+            *e = wap_event_create(Push_Message);
+
+    	if (octstr_compare(attr_name, octstr_imm("priority")) == 0)
             (**e).u.Push_Message.priority = 
                  (ret = parse_priority(attr_value)) >= 0 ? ret : 0;
         else if (octstr_compare(attr_name, octstr_imm("delivery-method")) == 0)
@@ -1322,8 +1323,7 @@ static int set_attribute_value(Octstr *element_name, Octstr *attr_value,
  */
 static int parse_code(Octstr *attr_value)
 {
-    long attr_as_number,
-         len;
+    long attr_as_number;
     size_t i;
     Octstr *ros;
 
@@ -1338,7 +1338,7 @@ static int parse_code(Octstr *attr_value)
 
     warning(0, "PAP COMPILER: parse_code: no such return code, reversing to"
                " x000 code");
-    len = octstr_parse_long(&attr_as_number, attr_value, 0, 10);
+    octstr_parse_long(&attr_as_number, attr_value, 0, 10);
     if (attr_as_number >= PAP_OK && attr_as_number < PAP_BAD_REQUEST) {
         attr_as_number = PAP_OK;
     } else if (attr_as_number >= PAP_BAD_REQUEST && 
