@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -68,6 +68,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <errno.h>
 
 struct CfgGroup {
     Octstr *name;
@@ -445,7 +446,6 @@ int cfg_read(Cfg *cfg)
     Octstr *filename; 
     CfgGroup *grp;
     long equals;
-    long lineno;
     long error_lineno;
     
     loc = loc_inc = NULL;
@@ -462,7 +462,6 @@ int cfg_read(Cfg *cfg)
     gwlist_insert(stack, 0, octstr_duplicate(cfg->filename)); 
 
     grp = NULL;
-    lineno = 0;
     error_lineno = 0;
     while (error_lineno == 0 && (loc = gwlist_extract_first(lines)) != NULL) { 
         octstr_strip_blanks(loc->line); 
@@ -500,8 +499,15 @@ int cfg_read(Cfg *cfg)
                     struct stat filestat;
 
                     /* check if included file is a directory */
-                    lstat(octstr_get_cstr(filename), &filestat);
-
+                    if (lstat(octstr_get_cstr(filename), &filestat) != 0) {
+                        error(errno, "lstat failed: couldn't stat `%s'", 
+                              octstr_get_cstr(filename));
+                        panic(0, "Failed to include `%s' "
+                              "(on line %ld of file %s). Aborting!",  
+                              octstr_get_cstr(filename), loc->line_no,  
+                              octstr_get_cstr(loc->filename)); 
+                    }
+                    
                     /* 
                      * is a directory, create a list with files of
                      * this directory and load all as part of the
