@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -64,7 +64,7 @@
 
 #include "gwlib/gwlib.h"
 
-#if HAVE_ICONV_H
+#if HAVE_ICONV
 #include <errno.h>
 #include <iconv.h>
 #endif
@@ -144,7 +144,6 @@ static const unsigned char gsm_to_latin1[128] = {
          'x',  'y',  'z', 0xe4, 0xf6, 0xf1, 0xfc, 0xe0    /* 120 - 127 */
 };
 
-
 /** 
  * Map GSM default alphabet characters to unicode codeposition.
  * The escape character, at position 27, is mapped to a NRP,
@@ -589,15 +588,17 @@ int charset_from_utf8(Octstr *utf8, Octstr **to, Octstr *charset_to)
 
 int charset_convert(Octstr* string, char* charset_from, char* charset_to)
 {
-#if HAVE_ICONV_H
+#if HAVE_ICONV
     char *from_buf, *to_buf, *pointer;
-    size_t inbytes, outbytes;
-    int ret;
+    size_t inbytesleft, outbytesleft, ret;
     iconv_t cd;
      
     if (!charset_from || !charset_to || !string) /* sanity check */
-         return -1;
-         
+        return -1;
+
+    if (octstr_len(string) < 1 || strcasecmp(charset_from, charset_to) == 0)
+        return 0; /* we are done, nothing to convert */
+        
     cd = iconv_open(charset_to, charset_from);
     /* Did I succeed in getting a conversion descriptor ? */
     if (cd == (iconv_t)(-1)) {
@@ -606,29 +607,65 @@ int charset_convert(Octstr* string, char* charset_from, char* charset_to)
               charset_from, charset_to);
         return -1; 
     }
+    
     from_buf = octstr_get_cstr(string);
+    inbytesleft = octstr_len(string);
     /* allocate max sized buffer, assuming target encoding may be 4 byte unicode */
-    inbytes = octstr_len(string);
-    outbytes = sizeof(char) * octstr_len(string) * 4;
-    pointer = to_buf = gw_malloc(outbytes + 1);
-    memset(to_buf, 0, outbytes + 1);
-    ret = iconv(cd, (char**)&from_buf, &inbytes, &pointer, &outbytes);
+    outbytesleft = inbytesleft * 4;
+    pointer = to_buf = gw_malloc(outbytesleft);
+
+    do {
+        ret = iconv(cd, (ICONV_CONST char**) &from_buf, &inbytesleft, &pointer, &outbytesleft);
+        if(ret == -1) {
+            long tmp;
+            /* the conversion failed somewhere */
+            switch(errno) {
+            case E2BIG: /* no space in output buffer */
+                debug("charset", 0, "outbuf to small, realloc.");
+                tmp = pointer - to_buf;
+                to_buf = gw_realloc(to_buf, tmp + inbytesleft * 4);
+                outbytesleft += inbytesleft * 4;
+                pointer = to_buf + tmp;
+                ret = 0;
+                break;
+            case EILSEQ: /* invalid multibyte sequence */
+            case EINVAL: /* incomplete multibyte sequence */
+                warning(0, "Invalid/Incomplete multibyte sequence at position %d, skeep it.",
+                        (int)(from_buf - octstr_get_cstr(string)));
+                /* skeep char and try next */
+                if (outbytesleft == 0) {
+                    /* buffer to small */
+                    tmp = pointer - to_buf;
+                    to_buf = gw_realloc(to_buf, tmp + inbytesleft * 4);
+                    outbytesleft += inbytesleft * 4;
+                    pointer = to_buf + tmp;
+                }
+                pointer[0] = from_buf[0];
+                pointer++;
+                from_buf++;
+                inbytesleft--;
+                outbytesleft--;
+                ret = 0;
+                break;
+            }
+        }
+    } while(inbytesleft && ret == 0); /* stop if error occurs and not handled above */
+    
     iconv_close(cd);
+    
     if (ret != -1) {
         /* conversion succeeded */
-        octstr_delete(string, 0, octstr_len(string));
+        octstr_truncate(string, 0);
         octstr_append_data(string, to_buf, pointer - to_buf);
-    if (ret)
-        debug("charset", 0, "charset_convert did %d non-reversible conversions", ret);
+        if (ret)
+            debug("charset", 0, "charset_convert did %ld non-reversible conversions", (long) ret);
         ret = 0;
-    } else {
-        error(0,"Failed to convert string from <%s> to <%s>, errno was <%d>",
-              charset_from, charset_to, errno);
-    }
+    } else
+        error(errno,"Failed to convert string from <%s> to <%s>.", charset_from, charset_to);
 
     if (errno == EILSEQ) {
         debug("charset_convert", 0, "Found an invalid multibyte sequence at position <%d>",
-              from_buf - octstr_get_cstr(string));     
+              (int)(from_buf - octstr_get_cstr(string)));
     }
     gw_free(to_buf);
     return ret;
