@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -297,7 +297,7 @@ void http_close_proxy(void);
 void http_set_interface(const Octstr *our_host);
 
 /**
- * Define timeout in seconds for which HTTP clint will wait for
+ * Define timeout in seconds for which HTTP client will wait for
  * response. Set -1 to disable timeouts.
  */
 void http_set_client_timeout(long timeout);
@@ -395,6 +395,12 @@ void *http_receive_result_real(HTTPCaller *caller, int *status, Octstr **final_u
 typedef struct HTTPClient HTTPClient;
 
 
+/**
+ * Define timeout in seconds for which HTTP server will wait for
+ * request. Set -1 to disable timeouts.
+ */
+void http_set_server_timeout(int port, long timeout);
+
 /*
  * Open an HTTP server at a given port. Return -1 for errors (invalid
  * port number, etc), 0 for OK. This will also start a background thread
@@ -469,6 +475,16 @@ void http_destroy_cgiargs(List *args);
  */
 Octstr *http_cgi_variable(List *list, char *name);
 
+/*
+ * Return METHOD used by client
+ */
+int http_method(HTTPClient *client);
+
+/*
+ * Return URL used by client
+ */
+Octstr *http_request_url(HTTPClient *client);
+
 
 /***********************************************************************
  * HTTP header interface.
