@@ -1,7 +1,7 @@
 /* ====================================================================
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -80,7 +80,8 @@
  * which makes passing of the &args a bit tricky 
  */
 #if (defined(__linux__) && (defined(__powerpc__) || defined(__s390__) || defined(__x86_64))) || \
-    (defined(__FreeBSD__) && defined(__amd64__))
+    (defined(__FreeBSD__) && defined(__amd64__)) || \
+    (defined(DARWIN) && defined(__x86_64__))
 #define VARGS(x)   (x)
 #define VALPARM(y) va_list y
 #define VALST(z)   (z)
@@ -141,7 +142,7 @@ static char is_safe[UCHAR_MAX + 1];
  * and discarding the lowest to bits to get rid of typical alignment
  * bits.
  */
-#define CSTR_TO_LONG(ptr)	(((long) ptr) >> 2)
+#define CSTR_TO_LONG(ptr)	(((unsigned long) ptr) >> 2)
 
 
 /*
@@ -206,9 +207,9 @@ static void urlcode_init(void)
     int i;
 
     unsigned char *safe = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-	"abcdefghijklmnopqrstuvwxyz-_.!~*'()";
+                          "abcdefghijklmnopqrstuvwxyz-_.!~*'()";
     for (i = 0; safe[i] != '\0'; ++i)
-	is_safe[safe[i]] = 1;
+        is_safe[safe[i]] = 1;
 }
 
 
@@ -348,7 +349,10 @@ long octstr_len(const Octstr *ostr)
 Octstr *octstr_copy_real(const Octstr *ostr, long from, long len, const char *file, long line,
                          const char *func)
 {
-    seems_valid(ostr);
+    if (ostr == NULL)
+        return octstr_create("");
+
+    seems_valid_real(ostr, file, line, func);
     gw_assert(from >= 0);
     gw_assert(len >= 0);
 
@@ -369,7 +373,7 @@ Octstr *octstr_duplicate_real(const Octstr *ostr, const char *file, long line,
 {
     if (ostr == NULL)
         return NULL;
-    seems_valid(ostr);
+    seems_valid_real(ostr, file, line, func);
     return octstr_create_from_data_trace(ostr->data, ostr->len, file, line, func);
 }
 
@@ -1004,7 +1008,7 @@ int octstr_str_ncompare(const Octstr *ostr, const char *str, long n)
 }
 
 
-int octstr_search_char(const Octstr *ostr, int ch, long pos)
+long octstr_search_char(const Octstr *ostr, int ch, long pos)
 {
     unsigned char *p;
 
@@ -1023,7 +1027,28 @@ int octstr_search_char(const Octstr *ostr, int ch, long pos)
 }
 
 
-int octstr_search_chars(const Octstr *ostr, const Octstr *chars, long pos)
+long octstr_rsearch_char(const Octstr *ostr, int ch, long pos)
+{
+    long i;
+
+    seems_valid(ostr);
+    gw_assert(ch >= 0);
+    gw_assert(ch <= UCHAR_MAX);
+    gw_assert(pos >= 0);
+
+    if (pos >= ostr->len)
+        return -1;
+
+    for (i = pos; i >= 0; i--) {
+        if (ostr->data[i] == ch)
+            return i;
+    }
+
+    return -1;
+}
+
+
+long octstr_search_chars(const Octstr *ostr, const Octstr *chars, long pos)
 {
     long i, j;
 
@@ -1041,7 +1066,7 @@ int octstr_search_chars(const Octstr *ostr, const Octstr *chars, long pos)
 }
 
 
-int octstr_search(const Octstr *haystack, const Octstr *needle, long pos)
+long octstr_search(const Octstr *haystack, const Octstr *needle, long pos)
 {
     int first;
 
@@ -1073,7 +1098,7 @@ int octstr_search(const Octstr *haystack, const Octstr *needle, long pos)
 }
 
 
-int octstr_case_search(const Octstr *haystack, const Octstr *needle, long pos)
+long octstr_case_search(const Octstr *haystack, const Octstr *needle, long pos)
 {
     long i, j;
     int c1, c2;
@@ -1100,7 +1125,7 @@ int octstr_case_search(const Octstr *haystack, const Octstr *needle, long pos)
     return -1;    
 }
 
-int octstr_case_nsearch(const Octstr *haystack, const Octstr *needle, long pos, long n)
+long octstr_case_nsearch(const Octstr *haystack, const Octstr *needle, long pos, long n)
 {
     long i, j;
     int c1, c2;
@@ -1128,6 +1153,40 @@ int octstr_case_nsearch(const Octstr *haystack, const Octstr *needle, long pos,
 }
 
 
+long octstr_str_search(const Octstr *haystack, const char *needle, long pos)
+{
+    int first;
+    int needle_len;
+
+    seems_valid(haystack);
+    gw_assert(pos >= 0);
+
+    /* Always "find" an empty string */
+    if (needle == NULL || needle[0] == '\0')
+        return 0;
+
+    needle_len = strlen(needle);
+
+    if (needle_len == 1)
+        return octstr_search_char(haystack, needle[0], pos);
+
+    /* For each occurrence of needle's first character in ostr,
+     * check if the rest of needle follows.  Stop if there are no
+     * more occurrences, or if the rest of needle can't possibly
+     * fit in the haystack. */
+    first = needle[0];
+    pos = octstr_search_char(haystack, first, pos);
+    while (pos >= 0 && haystack->len - pos >= needle_len) {
+        if (memcmp(haystack->data + pos,
+                   needle, needle_len) == 0)
+            return pos;
+        pos = octstr_search_char(haystack, first, pos + 1);
+    }
+
+    return -1;
+}
+
+
 int octstr_print(FILE *f, Octstr *ostr)
 {
     gw_assert(f != NULL);
@@ -1685,8 +1744,8 @@ void octstr_url_encode(Octstr *ostr)
 
 int octstr_url_decode(Octstr *ostr)
 {
-    unsigned char *string = ostr->data;
-    unsigned char *dptr = ostr->data;
+    unsigned char *string;
+    unsigned char *dptr;
     int code, code2, ret = 0;
 
     if (ostr == NULL)
@@ -1698,6 +1757,9 @@ int octstr_url_decode(Octstr *ostr)
     if (ostr->len == 0)
         return 0;
 
+    string = ostr->data;
+    dptr = ostr->data;
+
     do {
         if (*string == '%') {
             if (*(string + 1) == '\0' || *(string + 2) == '\0') {
@@ -1985,7 +2047,7 @@ static void octstr_dump_debug(const Octstr *ostr, int level)
  * the 3 log levels info(), warning() and error() that have the same
  * argument list.
  * We need to map the function calls via ## concatenation and revert
- * to the orginal function call by a define.
+ * to the original function call by a define.
  * The do-while loop emulates a function call.
  */
 
@@ -2454,18 +2516,24 @@ void octstr_format_append(Octstr *os, const char *fmt, ...)
 }
 
 
+/*
+ * Hash implementation ala Robert Sedgewick.
+ */
 unsigned long octstr_hash_key(Octstr *ostr)
 {
-    unsigned long key = 0;
-    long i;
+    unsigned long b    = 378551;
+    unsigned long a    = 63689;
+    unsigned long hash = 0;
+    unsigned long i    = 0;
+    unsigned long len = octstr_len(ostr);
+    const char *str = octstr_get_cstr(ostr);
 
-    if (ostr == NULL)
-	return 0;
+    for(i = 0; i < len; str++, i++) {
+        hash = hash*a+(*str);
+        a = a*b;
+    }
 
-    for (i = 0; i < octstr_len(ostr); i++)
-	key = key + octstr_get_char(ostr, i);
-
-    return key;
+    return (hash & 0x7FFFFFFF);
 }
 
 
@@ -2592,6 +2660,21 @@ void octstr_replace(Octstr *haystack, Octstr *needle, Octstr *repl)
     }
 }
 
+void octstr_replace_first(Octstr *haystack, Octstr *needle, Octstr *repl)
+{
+    int p = 0;
+    long len, repl_len;
+
+    len = octstr_len(needle);
+    repl_len = octstr_len(repl);
+
+    p = octstr_search(haystack, needle, p);
+    if (p != -1) {
+        octstr_delete(haystack, p, len);
+        octstr_insert(haystack, repl, p);
+    }
+}
+
 int octstr_symbolize(Octstr *ostr)
 {
     long len, i;
