@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -90,7 +90,7 @@ static const struct sockaddr_in empty_sockaddr_in;
 #endif
 
 
-int make_server_socket(int port, const char *interface_name )
+int make_server_socket(int port, const char *interface_name)
 {
     struct sockaddr_in addr;
     int s;
@@ -146,20 +146,20 @@ error:
 }
 
 
-int tcpip_connect_to_server(char *hostname, int port, const char *interface_name)
+int tcpip_connect_to_server(char *hostname, int port, const char *source_addr)
 {
 
-    return tcpip_connect_to_server_with_port(hostname, port, 0, interface_name);
+    return tcpip_connect_to_server_with_port(hostname, port, 0, source_addr);
 }
 
 
-int tcpip_connect_to_server_with_port(char *hostname, int port, int our_port, const char *interface_name)
+int tcpip_connect_to_server_with_port(char *hostname, int port, int our_port, const char *source_addr)
 {
     struct sockaddr_in addr;
     struct sockaddr_in o_addr;
     struct hostent hostinfo;
     struct hostent o_hostinfo;
-    int s;
+    int s, rc = -1, i;
     char *buff, *buff1;
 
     buff = buff1 = NULL;
@@ -175,30 +175,24 @@ int tcpip_connect_to_server_with_port(char *hostname, int port, int our_port, co
         goto error;
     }
 
-    addr = empty_sockaddr_in;
-    addr.sin_family = AF_INET;
-    addr.sin_port = htons(port);
-    addr.sin_addr = *(struct in_addr *) hostinfo.h_addr;
-
-    if (our_port > 0 || (interface_name != NULL && strcmp(interface_name, "*") != 0))  {
+    if (our_port > 0 || (source_addr != NULL && strcmp(source_addr, "*") != 0))  {
         int reuse;
 
         o_addr = empty_sockaddr_in;
         o_addr.sin_family = AF_INET;
         o_addr.sin_port = htons(our_port);
-	if (interface_name == NULL || strcmp(interface_name, "*") == 0)
-	    o_addr.sin_addr.s_addr = htonl(INADDR_ANY);
-	else {
-	    if (gw_gethostbyname(&o_hostinfo, interface_name, &buff1) == -1) {
-		error(errno, "gethostbyname failed");
-		goto error;
-	    }
-	    o_addr.sin_addr = *(struct in_addr *) o_hostinfo.h_addr;
-	}
+        if (source_addr == NULL || strcmp(source_addr, "*") == 0)
+            o_addr.sin_addr.s_addr = htonl(INADDR_ANY);
+        else {
+            if (gw_gethostbyname(&o_hostinfo, source_addr, &buff1) == -1) {
+                error(errno, "gethostbyname failed");
+                goto error;
+            }
+            o_addr.sin_addr = *(struct in_addr *) o_hostinfo.h_addr;
+        }
 
         reuse = 1;
-        if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &reuse,
-                       sizeof(reuse)) == -1) {
+        if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &reuse, sizeof(reuse)) == -1) {
             error(errno, "setsockopt failed before bind");
             goto error;
         }
@@ -208,18 +202,35 @@ int tcpip_connect_to_server_with_port(char *hostname, int port, int our_port, co
         }
     }
 
-    if (connect(s, (struct sockaddr *) &addr, sizeof(addr)) == -1) {
-        error(errno, "connect failed");
+    i = 0;
+    do {
+        Octstr *ip2;
+
+        addr = empty_sockaddr_in;
+        addr.sin_family = AF_INET;
+        addr.sin_port = htons(port);
+        addr.sin_addr = *(struct in_addr *) hostinfo.h_addr_list[i];
+
+        ip2 = gw_netaddr_to_octstr(AF_INET, &addr.sin_addr);
+
+        debug("gwlib.socket", 0, "Connecting to <%s>", octstr_get_cstr(ip2));
+
+        rc = connect(s, (struct sockaddr *) &addr, sizeof(addr));
+        if (rc == -1) {
+            error(errno, "connect to <%s> failed", octstr_get_cstr(ip2));
+        }
+        octstr_destroy(ip2);
+    } while (rc == -1 && hostinfo.h_addr_list[++i] != NULL);
+
+    if (rc == -1)
         goto error;
-    }
 
     gw_free(buff);
     gw_free(buff1);
     return s;
 
 error:
-    error(0, "error connecting to server `%s' at port `%d'",
-          hostname, port);
+    error(0, "error connecting to server `%s' at port `%d'", hostname, port);
     if (s >= 0)
         close(s);
     gw_free(buff);
@@ -227,97 +238,107 @@ error:
     return -1;
 }
 
-int tcpip_connect_nb_to_server(char *hostname, int port, const char *interface_name, int *done)
+int tcpip_connect_nb_to_server(char *hostname, int port, const char *source_addr, int *done)
 {
-  return tcpip_connect_nb_to_server_with_port(hostname, port, 0, interface_name, done);
+    return tcpip_connect_nb_to_server_with_port(hostname, port, 0, source_addr, done);
 }
 
-int tcpip_connect_nb_to_server_with_port(char *hostname, int port, int our_port, const char *interface_name, int *done) 
+int tcpip_connect_nb_to_server_with_port(char *hostname, int port, int our_port, const char *source_addr, int *done)
 {
-  struct sockaddr_in addr;
-  struct sockaddr_in o_addr;
-  struct hostent hostinfo;
-  struct hostent o_hostinfo;
-  int s;
-  int flags,rc;
-  char *buff, *buff1;
+    struct sockaddr_in addr;
+    struct sockaddr_in o_addr;
+    struct hostent hostinfo;
+    struct hostent o_hostinfo;
+    int s, flags, rc = -1, i;
+    char *buff, *buff1;
 
-  *done = 1;
-  buff = buff1 = NULL;
+    *done = 1;
+    buff = buff1 = NULL;
 
-  s = socket(PF_INET, SOCK_STREAM, 0);
-  if (s == -1) {
-    error(errno, "Couldn't create new socket.");
-    goto error;
-  }
-
-  if (gw_gethostbyname(&hostinfo, hostname, &buff) == -1) {
-    error(errno, "gethostbyname failed");
-    goto error;
-  }
-
-  addr = empty_sockaddr_in;
-  addr.sin_family = AF_INET;
-  addr.sin_port = htons(port);
-  addr.sin_addr = *(struct in_addr *) hostinfo.h_addr;
-
-  if (our_port > 0 || (interface_name != NULL && strcmp(interface_name, "*") != 0)) {
-    int reuse;
-
-    o_addr = empty_sockaddr_in;
-    o_addr.sin_family = AF_INET;
-    o_addr.sin_port = htons(our_port);
-    if (interface_name == NULL || strcmp(interface_name, "*") == 0)
-      o_addr.sin_addr.s_addr = htonl(INADDR_ANY);
-    else {
-      if (gw_gethostbyname(&o_hostinfo, interface_name, &buff1) == -1) {
-	error(errno, "gethostbyname failed");
-	goto error;
-      }
-      o_addr.sin_addr = *(struct in_addr *) o_hostinfo.h_addr;
+    s = socket(PF_INET, SOCK_STREAM, 0);
+    if (s == -1) {
+        error(errno, "Couldn't create new socket.");
+        goto error;
     }
 
-    reuse = 1;
-    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &reuse,
-		   sizeof(reuse)) == -1) {
-      error(errno, "setsockopt failed before bind");
-      goto error;
+    if (gw_gethostbyname(&hostinfo, hostname, &buff) == -1) {
+        error(errno, "gethostbyname failed");
+        goto error;
     }
-    if (bind(s, (struct sockaddr *) &o_addr, sizeof(o_addr)) == -1) {
-      error(errno, "bind to local port %d failed", our_port);
-      goto error;
+
+    if (our_port > 0 || (source_addr != NULL && strcmp(source_addr, "*") != 0)) {
+        int reuse;
+
+        o_addr = empty_sockaddr_in;
+        o_addr.sin_family = AF_INET;
+        o_addr.sin_port = htons(our_port);
+        if (source_addr == NULL || strcmp(source_addr, "*") == 0)
+            o_addr.sin_addr.s_addr = htonl(INADDR_ANY);
+        else {
+            if (gw_gethostbyname(&o_hostinfo, source_addr, &buff1) == -1) {
+                error(errno, "gethostbyname failed");
+                goto error;
+            }
+            o_addr.sin_addr = *(struct in_addr *) o_hostinfo.h_addr;
+        }
+
+        reuse = 1;
+        if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &reuse, sizeof(reuse)) == -1) {
+            error(errno, "setsockopt failed before bind");
+            goto error;
+        }
+        if (bind(s, (struct sockaddr *) &o_addr, sizeof(o_addr)) == -1) {
+            error(errno, "bind to local port %d failed", our_port);
+            goto error;
+        }
     }
-  }
 
-  flags = fcntl(s, F_GETFL, 0);
-  fcntl(s, F_SETFL, flags | O_NONBLOCK);
+    flags = fcntl(s, F_GETFL, 0);
+    fcntl(s, F_SETFL, flags | O_NONBLOCK);
 
-  if ((rc = connect(s, (struct sockaddr *) &addr, sizeof(addr))) < 0) {
-    if (errno != EINPROGRESS) {
-      error(errno, "nonblocking connect failed");
-      goto error;
+    i = 0;
+    do {
+        Octstr *ip2;
+
+        addr = empty_sockaddr_in;
+        addr.sin_family = AF_INET;
+        addr.sin_port = htons(port);
+        addr.sin_addr = *(struct in_addr *) hostinfo.h_addr_list[i];
+
+        ip2 = gw_netaddr_to_octstr(AF_INET, &addr.sin_addr);
+
+        debug("gwlib.socket", 0, "Connecting nonblocking to <%s>", octstr_get_cstr(ip2));
+
+        if ((rc = connect(s, (struct sockaddr *) &addr, sizeof(addr))) < 0) {
+            if (errno != EINPROGRESS) {
+                error(errno, "nonblocking connect to <%s> failed", octstr_get_cstr(ip2));
+            }
+        }
+        octstr_destroy(ip2);
+    } while (rc == -1 && errno != EINPROGRESS && hostinfo.h_addr_list[++i] != NULL);
+
+    if (rc == -1 && errno != EINPROGRESS)
+        goto error;
+
+    /* May be connected immediatly
+     * (if we connecting to localhost for example)
+     */
+    if (rc == 0) {
+        *done = 0;
     }
-  }
-  
-  /* May be connected immediatly
-   * (if we connecting to localhost for example) */
-  if (rc == 0) { 
-    *done = 0;
-  }
 
-  gw_free(buff);
-  gw_free(buff1);
+    gw_free(buff);
+    gw_free(buff1);
 
-  return s;
+    return s;
 
- error:
-  error(0, "error connecting to server `%s' at port `%d'",
-	hostname, port);
-  if (s >= 0)
-    close(s);
-  gw_free(buff);
-  gw_free(buff1);
-  return -1;
+error:
+    error(0, "error connecting to server `%s' at port `%d'", hostname, port);
+    if (s >= 0)
+        close(s);
+    gw_free(buff);
+    gw_free(buff1);
+    return -1;
 }
 
 
@@ -370,6 +391,18 @@ int socket_set_blocking(int fd, int blocking)
 }
 
 
+int socket_set_nodelay(int fd, int on)
+{
+    int rc;
+
+    rc = setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof (on));
+    if (rc == -1)
+        error(errno, "Unable set TCP_NODELAY(%d)", on);
+
+    return rc;
+}
+
+
 int read_available(int fd, long wait_usec)
 {
     fd_set rf;
@@ -442,7 +475,7 @@ int udp_client_socket(void)
 }
 
 
-int udp_bind(int port, const char *interface_name)
+int udp_bind(int port, const char *source_addr)
 {
     int s;
     struct sockaddr_in sa;
@@ -458,10 +491,10 @@ int udp_bind(int port, const char *interface_name)
     sa = empty_sockaddr_in;
     sa.sin_family = AF_INET;
     sa.sin_port = htons(port);
-    if (strcmp(interface_name, "*") == 0)
+    if (strcmp(source_addr, "*") == 0)
         sa.sin_addr.s_addr = htonl(INADDR_ANY);
     else {
-        if (gw_gethostbyname(&hostinfo, interface_name, &buff) == -1) {
+        if (gw_gethostbyname(&hostinfo, source_addr, &buff) == -1) {
             error(errno, "gethostbyname failed");
             gw_free(buff);
             return -1;
@@ -547,17 +580,22 @@ int udp_sendto(int s, Octstr *datagram, Octstr *addr)
 
 
 int udp_recvfrom(int s, Octstr **datagram, Octstr **addr)
+{
+    return udp_recvfrom_flags(s, datagram, addr, 0);
+}
+
+
+int udp_recvfrom_flags(int s, Octstr **datagram, Octstr **addr, int sockrcvflags)
 {
     struct sockaddr_in sa;
-    int salen;
+    socklen_t salen;
     char *buf;
     int bytes;
 
     buf = gw_malloc(UDP_PACKET_MAX_SIZE);
 
     salen = sizeof(sa);
-    bytes = recvfrom(s, buf, UDP_PACKET_MAX_SIZE, 0,
-                     (struct sockaddr *) &sa, &salen);
+    bytes = recvfrom(s, buf, UDP_PACKET_MAX_SIZE, sockrcvflags, (struct sockaddr *) &sa, &salen);
     if (bytes == -1) {
         if (errno != EAGAIN)
             error(errno, "Couldn't receive UDP packet");
@@ -567,7 +605,7 @@ int udp_recvfrom(int s, Octstr **datagram, Octstr **addr)
 
     *datagram = octstr_create_from_data(buf, bytes);
     *addr = octstr_create_from_data((char *) &sa, salen);
-    
+
     gw_free(buf);
 
     return 0;
@@ -636,48 +674,33 @@ void socket_shutdown(void)
 }
 
 
-static Octstr *gw_netaddr_to_octstr4(unsigned char *src)
-{
-    return octstr_format("%d.%d.%d.%d", src[0], src[1], src[2], src[3]);
-}
-
-
-#ifdef AF_INET6
-static Octstr *gw_netaddr_to_octstr6(unsigned char *src)
-{
-    return octstr_format(
-	    	"%x:%x:%x:%x:"
-		"%x:%x:%x:%x:"
-		"%x:%x:%x:%x:"
-		"%x:%x:%x:%x",
-	         src[0],  src[1],  src[2],  src[3], 
-		 src[4],  src[5],  src[6],  src[7], 
-		 src[8],  src[9], src[10], src[11], 
-		src[12], src[13], src[14], src[15]);
-}
-#endif
-
 Octstr *gw_netaddr_to_octstr(int af, void *src)
 {
     switch (af) {
-    case AF_INET:
-	return gw_netaddr_to_octstr4(src);
+    case AF_INET: {
+        char straddr[INET_ADDRSTRLEN];
+        inet_ntop(AF_INET, src, straddr, sizeof(straddr));
+        return octstr_create(straddr);
+    }
 
 #ifdef AF_INET6
-    case AF_INET6:
-	return gw_netaddr_to_octstr6(src);
+    case AF_INET6: {
+        char straddr[INET6_ADDRSTRLEN];
+        inet_ntop(AF_INET6, src, straddr, sizeof(straddr));
+        return octstr_create(straddr);
+    }
 #endif
 
     default:
-	return NULL;
-    } 
+        return NULL;
+    }
 }
 
 
 int gw_accept(int fd, Octstr **client_addr)
 {
     struct sockaddr_in addr;
-    int addrlen;
+    socklen_t addrlen;
     int new_fd;
 
     if (gwthread_pollfd(fd, POLLIN, -1.0) != POLLIN) {
