@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -65,6 +65,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 
 #include "gw-config.h"
 
@@ -85,23 +86,23 @@ Octstr *get_official_name(void);
 Octstr *get_official_ip(void);
 
 /* Open a server socket. Return -1 for error, >= 0 socket number for OK.*/
-int make_server_socket(int port, const char *interface_name);
+int make_server_socket(int port, const char *source_addr);
 
 /* Open a client socket. */
-int tcpip_connect_to_server(char *hostname, int port, const char *interface_name);
+int tcpip_connect_to_server(char *hostname, int port, const char *source_addr);
 
 /* As above, but binds our end to 'our_port' */
 int tcpip_connect_to_server_with_port(char *hostname, int port, int our_port,
-		const char *interface_name);
+		const char *source_addr);
 
 /* Open a client socket in nonblocking mode, done is 0 if socket 
    connected  immediatly, overwise done is 1 */
-int tcpip_connect_nb_to_server(char *hostname, int port, const char *interface_name,
+int tcpip_connect_nb_to_server(char *hostname, int port, const char *source_addr,
                                int *done);
 
 /* As above, but binds our end to 'our_port' */
 int tcpip_connect_nb_to_server_with_port(char *hostname, int port, int our_port,
-                                         const char *interface_name, int *done);
+                                         const char *source_addr, int *done);
 
 /* Write string to socket. */
 int write_to_socket(int socket, char *str);
@@ -110,6 +111,11 @@ int write_to_socket(int socket, char *str);
  * 0 for success. */
 int socket_set_blocking(int socket, int blocking);
 
+/* Diable or enable TCP_NODELAY.
+ * Return -1 for error, 0 for success.
+ */
+int socket_set_nodelay(int fd, int on);
+
 /* Check if there is something to be read in 'fd'. Return 1 if there
  * is data, 0 otherwise, -1 on error */
 int read_available(int fd, long wait_usec);
@@ -119,7 +125,7 @@ int read_available(int fd, long wait_usec);
  * Create a UDP socket for receiving from clients. Return -1 for failure,
  * a socket file descriptor >= 0 for OK.
  */
-int udp_bind(int port, const char *interface_name);
+int udp_bind(int port, const char *source_addr);
 
 
 /*
@@ -159,6 +165,7 @@ int udp_sendto(int s, Octstr *datagram, Octstr *addr);
 /*
  * Receive a UDP message from a client.
  */
+int udp_recvfrom_flags(int s, Octstr **datagram, Octstr **addr, int sockrcvflags);
 int udp_recvfrom(int s, Octstr **datagram, Octstr **addr);
 
 
