@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 2001-2014 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -74,14 +74,66 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <sys/utsname.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
+#include <libgen.h>
+
+#if HAVE_UCONTEXT
+#include <sys/ucontext.h>
+#endif
 
 #include "gwlib.h"
 
+#if HAVE_BACKTRACE
+#include <execinfo.h> /*backtrace */
+#endif
+
+/* Headers required for the version dump. */
+#if defined(HAVE_LIBSSL) || defined(HAVE_WTLS_OPENSSL) 
+#include <openssl/opensslv.h>
+#endif
+#ifdef HAVE_MYSQL 
+#include <mysql_version.h>
+#include <mysql.h>
+#endif
+/*
+ * PostgreSQL drives us in a mess here slights. Even
+ * if our own configure run didn't detect openssl and hence
+ * gw-config.h has no HAVE_LIBSSL set, it is generally set
+ * on most distro in <pg_config.h>, so we end up in unresolved
+ * items at some point. We trick this by undef it again here.
+ */
+#ifdef HAVE_PGSQL
+# ifndef HAVE_LIBSSL
+# define UNDEF_LIBSSL 1
+# endif
+#include <libpq-fe.h>
+#include <pg_config.h>
+# ifdef UNDEF_LIBSSL
+# undef HAVE_LIBSSL
+# undef UNDEF_LIBSSL
+# endif
+#endif  /* HAVE_PGSQL */
+#ifdef HAVE_SQLITE 
+#include <sqlite.h>
+#endif
+#ifdef HAVE_SQLITE3 
+#include <sqlite3.h>
+#endif
+#ifdef HAVE_ORACLE 
+#include <oci.h>
+#endif
+#ifdef HAVE_REDIS 
+#include <hiredis.h>
+#endif
+
+
 /* pid of child process when parachute is used */
 static pid_t child_pid = -1;
+/* pid of pid file owner */
+static pid_t pidfile_owner_pid = -1;
 /* saved child signal handlers */
 static struct sigaction child_actions[32];
 /* just a flag that child signal handlers are stored */
@@ -91,6 +143,35 @@ static char *pid_file = NULL;
 static volatile sig_atomic_t parachute_shutdown = 0;
 
 
+static void fatal_handler(int sig, siginfo_t *info, void *secret)
+{
+#ifdef HAVE_BACKTRACE
+    void *trace[50];
+#ifdef REG_EIP
+    ucontext_t *uc = (ucontext_t*)secret;
+#endif
+    size_t size;
+#endif    
+    struct sigaction act;
+    
+    act.sa_handler = SIG_DFL;
+    sigemptyset(&act.sa_mask);
+    act.sa_flags = 0;
+    sigaction(sig, &act, NULL);
+    
+#ifdef HAVE_BACKTRACE
+    size = backtrace(trace, sizeof(trace) / sizeof(void*));
+#ifdef REG_EIP
+    /* overwrite sigaction with caller's address */
+    trace[1] = (void *) uc->uc_mcontext.gregs[REG_EIP];
+#endif
+    gw_backtrace(trace, size, 0);
+#endif
+    
+    raise(sig);
+}
+
+
 static void parachute_sig_handler(int signum)
 {
     info(0, "Signal %d received, forward to child pid (%ld)", signum, (long) child_pid);
@@ -144,6 +225,7 @@ static void parachute_init_signals(int child)
         sigaction(SIGTTIN, &sa, NULL);
         sigaction(SIGTSTP, &sa, NULL);
         child_actions_init = 1;
+        init_fatal_signals();
     }
     else
         panic(0, "Child process signal handlers not initialized before.");
@@ -162,7 +244,8 @@ static int is_executable(const char *filename)
         return 0;
     }
     /* others has exec permission */
-    if (S_IXOTH & buf.st_mode) return 1;
+    if (S_IXOTH & buf.st_mode)
+        return 1;
     /* group has exec permission */
     if ((S_IXGRP & buf.st_mode) && buf.st_gid == getgid())
         return 1;
@@ -329,7 +412,7 @@ static void write_pid_file(void)
     if (!file)
         panic(errno, "Could not open file-stream `%s'", pid_file);
 
-    fprintf(file, "%ld\n", (long) getpid());
+    fprintf(file, "%ld\n", (long) (pidfile_owner_pid = getpid()));
     fclose(file);
 }
 
@@ -338,14 +421,24 @@ static void remove_pid_file(void)
     if (!pid_file)
         return;
 
-    /* ensure we don't called from child process */
-    if (child_pid == 0)
+    /* ensure that only pidfile owner can remove it */
+    if (pidfile_owner_pid != getpid())
         return;
 
-    if (-1 == unlink(pid_file))
+    if (-1 == unlink(pid_file)) {
+        int initdone = gwlib_initialized();
+        /* we are called at exit so gwlib may be shutdown already, init again */
+        if (!initdone) {
+            gwlib_init();
+            log_set_syslog("kannel", 0);
+        }
         error(errno, "Could not unlink pid-file `%s'", pid_file);
+        if (!initdone)
+            gwlib_shutdown();
+    }
 }
 
+
 static int change_user(const char *user)
 {
     struct passwd *pass;
@@ -364,9 +457,11 @@ static int change_user(const char *user)
         return -1;
     }
 
-    if (initgroups(user, -1) == -1) {
+#ifdef HAVE_INITGROUPS
+    if (initgroups(user, pass->pw_gid) == -1) {
         error(errno, "Could not set supplementary group ID's.");
     }
+#endif
     
     if (-1 == setuid(pass->pw_uid)) {
         error(errno, "Could not change user id from %ld to %ld.", (long) getuid(), (long) pass->pw_uid);
@@ -381,7 +476,6 @@ static int change_user(const char *user)
  */
 
 
-
 MultibyteInt get_variable_value(Octet *source, int *len)
 {
     MultibyteInt retval = 0;
@@ -415,6 +509,7 @@ int write_variable_value(MultibyteInt value, Octet *dest)
     return loc;
 }
 
+
 Octet reverse_octet(Octet source)
 {
     Octet	dest;
@@ -431,6 +526,105 @@ Octet reverse_octet(Octet source)
 }
 
 
+void init_fatal_signals()
+{
+    /* install fatal signal handler */
+    struct sigaction act;
+    /* set segfault handler */
+    sigemptyset(&act.sa_mask);
+    act.sa_sigaction = fatal_handler;
+    act.sa_flags = SA_SIGINFO;
+    sigaction(SIGSEGV, &act, NULL);
+}
+
+
+void report_versions(const char *boxname)
+{
+    Octstr *os;
+    
+    os = version_report_string(boxname);
+    debug("gwlib.gwlib", 0, "%s", octstr_get_cstr(os));
+    octstr_destroy(os);
+}
+
+
+Octstr *version_report_string(const char *boxname)
+{
+    struct utsname u;
+
+    uname(&u);
+    return octstr_format(GW_NAME " %s version `%s'.\nBuild `%s', compiler `%s'.\n"
+                         "System %s, release %s, version %s, machine %s.\n"
+             "Hostname %s, IP %s.\n"
+             "Libxml version %s.\n"
+#ifdef HAVE_LIBSSL
+             "Using "
+#ifdef HAVE_WTLS_OPENSSL
+             "WTLS library "
+#endif
+             "%s.\n"
+#endif
+#ifdef HAVE_MYSQL
+             "Compiled with MySQL %s, using MySQL %s.\n"
+#endif
+#ifdef HAVE_PGSQL
+             "Compiled with PostgreSQL %s.\n"
+#endif
+#ifdef HAVE_SDB
+             "Using LibSDB %s.\n"
+#endif
+#if defined(HAVE_SQLITE) || defined(HAVE_SQLITE3)
+             "Using SQLite %s.\n"
+#endif
+#ifdef HAVE_ORACLE
+#if defined(OCI_MAJOR_VERSION) && defined(OCI_MINOR_VERSION)
+             "Using Oracle OCI %d.%d.\n"
+#else
+             "Using Oracle OCI.\n"
+#endif
+#endif
+#ifdef HAVE_REDIS
+             "Using hiredis API %d.%d.%d\n"
+#endif
+             "Using %s malloc.\n",
+             boxname, GW_VERSION,
+#ifdef __GNUC__ 
+             (__DATE__ " " __TIME__) ,
+             __VERSION__,
+#else 
+             "unknown" , "unknown",
+#endif 
+             u.sysname, u.release, u.version, u.machine,
+             octstr_get_cstr(get_official_name()),
+             octstr_get_cstr(get_official_ip()),
+             LIBXML_DOTTED_VERSION,
+#ifdef HAVE_LIBSSL
+             OPENSSL_VERSION_TEXT,
+#endif
+#ifdef HAVE_MYSQL
+             MYSQL_SERVER_VERSION, mysql_get_client_info(),
+#endif
+#ifdef HAVE_PGSQL
+             PG_VERSION,
+#endif
+#ifdef HAVE_SDB
+             LIBSDB_VERSION,
+#endif
+#if defined(HAVE_SQLITE) || defined(HAVE_SQLITE3)
+             SQLITE_VERSION,
+#endif
+#ifdef HAVE_ORACLE
+#if defined(OCI_MAJOR_VERSION) && defined(OCI_MINOR_VERSION)
+             OCI_MAJOR_VERSION, OCI_MINOR_VERSION,
+#endif
+#endif
+#ifdef HAVE_REDIS
+			 HIREDIS_MAJOR, HIREDIS_MINOR, HIREDIS_PATCH,
+#endif
+             octstr_get_cstr(gwmem_type()));
+}
+
+
 int get_and_set_debugs(int argc, char **argv,
 		       int (*find_own) (int index, int argc, char **argv))
 {
@@ -492,6 +686,11 @@ int get_and_set_debugs(int argc, char **argv,
         } else if (strcmp(argv[i], "-g")==0 || strcmp(argv[i], "--generate")==0) {
             cfg_dump_all();
             exit(0);
+        } else if (strcmp(argv[i], "--version")==0) {
+            Octstr *version = version_report_string(basename(argv[0]));
+            printf("%s", octstr_get_cstr(version));
+            octstr_destroy(version);
+            exit(0);
         } else if (strcmp(argv[i],"--")==0) {
             i++;
             break;
@@ -552,6 +751,9 @@ int get_and_set_debugs(int argc, char **argv,
     if (debug_places != NULL)
 	    info(0, "Debug places: `%s'", debug_places);
 
+
+    init_fatal_signals();
+    
     return i;
 }
 
