@@ -1,7 +1,64 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2014 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * wsp_session.c - Implement WSP session oriented service
  *
  * Lars Wirzenius
+ * Stipe Tolj
  */
 
 
@@ -84,9 +141,10 @@ static WSPPushMachine *push_machine_create(WSPMachine *session, long id);
 static void push_machine_destroy(void *p);
 
 static char *state_name(WSPState state);
-static long next_wsp_session_id(void);
+static unsigned long next_wsp_session_id(void);
 
 static List *make_capabilities_reply(WSPMachine *m);
+static List *make_reply_headers(WSPMachine *m);
 static Octstr *make_connectreply_pdu(WSPMachine *m);
 static Octstr *make_resume_reply_pdu(WSPMachine *m, List *headers);
 static WSP_PDU *make_confirmedpush_pdu(WAPEvent *e);
@@ -123,7 +181,8 @@ static void confirm_push(WSPPushMachine *machine);
 
 static void main_thread(void *);
 static int id_belongs_to_session (void *, void *);
-
+static int wsp_encoding_string_to_version(Octstr *enc);
+static Octstr *wsp_encoding_version_to_string(int version);
 
 
 /***********************************************************************
@@ -135,9 +194,9 @@ void wsp_session_init(wap_dispatch_func_t *responder_dispatch,
                       wap_dispatch_func_t *initiator_dispatch,
                       wap_dispatch_func_t *application_dispatch,
                       wap_dispatch_func_t *push_ota_dispatch) {
-	queue = list_create();
-	list_add_producer(queue);
-	session_machines = list_create();
+	queue = gwlist_create();
+	gwlist_add_producer(queue);
+	session_machines = gwlist_create();
 	session_id_counter = counter_create();
 	dispatch_to_wtp_resp = responder_dispatch;
 	dispatch_to_wtp_init = initiator_dispatch;
@@ -152,14 +211,14 @@ void wsp_session_init(wap_dispatch_func_t *responder_dispatch,
 void wsp_session_shutdown(void) {
 	gw_assert(run_status == running);
 	run_status = terminating;
-	list_remove_producer(queue);
+	gwlist_remove_producer(queue);
 	gwthread_join_every(main_thread);
 
-	list_destroy(queue, wap_event_destroy_item);
+	gwlist_destroy(queue, wap_event_destroy_item);
 
 	debug("wap.wsp", 0, "WSP: %ld session machines left.",
-		list_len(session_machines));
-	list_destroy(session_machines, machine_destroy);
+		gwlist_len(session_machines));
+	gwlist_destroy(session_machines, machine_destroy);
 
 	counter_destroy(session_id_counter);
         wsp_strings_shutdown();
@@ -168,7 +227,7 @@ void wsp_session_shutdown(void) {
 
 void wsp_session_dispatch_event(WAPEvent *event) {
 	wap_event_assert(event);
-	list_produce(queue, event);
+	gwlist_produce(queue, event);
 }
 
 
@@ -182,7 +241,7 @@ static void main_thread(void *arg) {
 	WSPMachine *sm;
 	WSP_PDU *pdu;
 	
-	while (run_status == running && (e = list_consume(queue)) != NULL) {
+	while (run_status == running && (e = gwlist_consume(queue)) != NULL) {
 		wap_event_assert(e);
 		switch (e->type) {
 		case TR_Invoke_Ind:
@@ -302,7 +361,7 @@ static WSPMachine *find_session_machine(WAPEvent *event, WSP_PDU *pdu) {
 		/* Pass to session identified by session id, not
 		 * the address tuple. */
 		session_id = pdu->u.Resume.sessionid;
-		sm = list_search(session_machines, &session_id,
+		sm = gwlist_search(session_machines, &session_id,
 				find_by_session_id);
 		if (sm == NULL) {
 			/* No session; TR-Abort.req(DISCONNECT) */
@@ -314,7 +373,7 @@ static WSPMachine *find_session_machine(WAPEvent *event, WSP_PDU *pdu) {
 	 * TR-Invoke.ind here by ignoring them; this seems to be
 	 * an omission in the spec table. */
 	} else if (event->type == TR_Invoke_Ind) {
-		sm = list_search(session_machines, tuple,
+		sm = gwlist_search(session_machines, tuple,
 				 transaction_belongs_to_session);
 		if (sm == NULL && (event->u.TR_Invoke_Ind.tcl == 1 ||
 				event->u.TR_Invoke_Ind.tcl == 2)) {
@@ -325,10 +384,10 @@ static WSPMachine *find_session_machine(WAPEvent *event, WSP_PDU *pdu) {
 	 * do those later, after we've tried to handle them. */
 	} else {
 		if (session_id != -1) {
-			sm = list_search(session_machines, &session_id,
+			sm = gwlist_search(session_machines, &session_id,
 				find_by_session_id);
 		} else {
-			sm = list_search(session_machines, tuple,
+			sm = gwlist_search(session_machines, tuple,
 				transaction_belongs_to_session);
 		}
 		/* The table doesn't really say what we should do with
@@ -431,9 +490,10 @@ static WSPMachine *machine_create(void) {
 	#define OCTSTR(name) p->name = NULL;
 	#define HTTPHEADERS(name) p->name = NULL;
 	#define ADDRTUPLE(name) p->name = NULL;
-	#define MACHINESLIST(name) p->name = list_create();
+	#define MACHINESLIST(name) p->name = gwlist_create();
 	#define CAPABILITIES(name) p->name = NULL;
-	#define COOKIES(name) p->name = NULL;
+	#define COOKIES(name) p->name = gwlist_create();
+	#define REFERER(name) p->name = NULL;
 	#define MACHINE(fields) fields
 	#include "wsp_server_session_machine.def"
 	
@@ -448,28 +508,28 @@ static WSPMachine *machine_create(void) {
 	 * to get events than old machines are, so this speeds up the linear
 	 * search, and 2) we want the newest machine to get any method
 	 * invokes that come through before the Connect is established. */
-	list_insert(session_machines, 0, p);
+	gwlist_insert(session_machines, 0, p);
 
 	return p;
 }
 
 
 static void destroy_methodmachines(List *machines) {
-	if (list_len(machines) > 0) {
+	if (gwlist_len(machines) > 0) {
 		warning(0, "Destroying WSP session with %ld active methods\n",
-			list_len(machines));
+			gwlist_len(machines));
 	}
 
-	list_destroy(machines, method_machine_destroy);
+	gwlist_destroy(machines, method_machine_destroy);
 }
 
 static void destroy_pushmachines(List *machines) {
-	if (list_len(machines) > 0) {
+	if (gwlist_len(machines) > 0) {
 		warning(0, "Destroying WSP session with %ld active pushes\n",
-			list_len(machines));
+			gwlist_len(machines));
 	}
 
-	list_destroy(machines, push_machine_destroy);
+	gwlist_destroy(machines, push_machine_destroy);
 }
 
 static void machine_destroy(void *pp) {
@@ -477,7 +537,7 @@ static void machine_destroy(void *pp) {
 	
 	p = pp;
 	debug("wap.wsp", 0, "Destroying WSPMachine %p", pp);
-	list_delete_equal(session_machines, p);
+	gwlist_delete_equal(session_machines, p);
 
 	#define INTEGER(name) p->name = 0;
 	#define OCTSTR(name) octstr_destroy(p->name);
@@ -486,6 +546,7 @@ static void machine_destroy(void *pp) {
 	#define MACHINESLIST(name) destroy_##name(p->name);
 	#define CAPABILITIES(name) wsp_cap_destroy_list(p->name);
 	#define COOKIES(name) cookies_destroy(p->name);
+	#define REFERER(name) octstr_destroy(p->name);
 	#define MACHINE(fields) fields
 	#include "wsp_server_session_machine.def"
 	gw_free(p);
@@ -537,7 +598,7 @@ WAPEvent *current_event, WSP_PDU *pdu) {
 end:
 	if (msm->state == NULL_METHOD) {
 		method_machine_destroy(msm);
-		list_delete_equal(sm->methodmachines, msm);
+		gwlist_delete_equal(sm->methodmachines, msm);
 	}
 }
 
@@ -559,7 +620,7 @@ static WSPMethodMachine *method_machine_create(WSPMachine *sm,
 	msm->addr_tuple = wap_addr_tuple_duplicate(sm->addr_tuple);
 	msm->session_id = sm->session_id;
 
-	list_append(sm->methodmachines, msm);
+	gwlist_append(sm->methodmachines, msm);
 
 	return msm;
 }
@@ -619,7 +680,7 @@ static void handle_push_event(WSPMachine *sm, WSPPushMachine *pm,
 end:
         if (pm->state == SERVER_PUSH_NULL_STATE) {
 		push_machine_destroy(pm);
-		list_delete_equal(sm->pushmachines, pm);
+		gwlist_delete_equal(sm->pushmachines, pm);
 	}
 }
 
@@ -642,7 +703,7 @@ static WSPPushMachine *push_machine_create(WSPMachine *sm,
 	m->addr_tuple = wap_addr_tuple_duplicate(sm->addr_tuple);
 	m->session_id = sm->session_id;
 
-	list_append(sm->pushmachines, m);
+	gwlist_append(sm->pushmachines, m);
 
 	return m;      
 }
@@ -685,7 +746,7 @@ static char *state_name(WSPState state) {
 }
 
 
-static long next_wsp_session_id(void) {
+static unsigned long next_wsp_session_id(void) {
 	return counter_increase(session_id_counter);
 }
 
@@ -695,8 +756,8 @@ static void sanitize_capabilities(List *caps, WSPMachine *m) {
 	Capability *cap;
 	unsigned long ui;
 
-	for (i = 0; i < list_len(caps); i++) {
-		cap = list_get(caps, i);
+	for (i = 0; i < gwlist_len(caps); i++) {
+		cap = gwlist_get(caps, i);
 
 		/* We only know numbered capabilities.  Let the application
 		 * layer negotiate whatever it wants for unknown ones. */
@@ -755,7 +816,7 @@ static void sanitize_capabilities(List *caps, WSPMachine *m) {
 	bad_cap:
 		error(0, "WSP: Found illegal value in capabilities reply.");
 		wsp_cap_dump(cap);
-		list_delete(caps, i, 1);
+		gwlist_delete(caps, i, 1);
 		i--;
 		wsp_cap_destroy(cap);
 		continue;
@@ -780,7 +841,7 @@ static void reply_known_capabilities(List *caps, List *req, WSPMachine *m) {
 		octstr_append_uintvar(data, m->client_SDU_size);
 		cap = wsp_cap_create(WSP_CAPS_CLIENT_SDU_SIZE,
 			NULL, data);
-		list_append(caps, cap);
+		gwlist_append(caps, cap);
 	}
 
 	if (wsp_cap_count(caps, WSP_CAPS_SERVER_SDU_SIZE, NULL) == 0) {
@@ -793,7 +854,7 @@ static void reply_known_capabilities(List *caps, List *req, WSPMachine *m) {
 		data = octstr_create("");
 		octstr_append_uintvar(data, ui);
 		cap = wsp_cap_create(WSP_CAPS_SERVER_SDU_SIZE, NULL, data);
-		list_append(caps, cap);
+		gwlist_append(caps, cap);
 	}
 
 	/* Currently we cannot handle any protocol options */
@@ -801,7 +862,7 @@ static void reply_known_capabilities(List *caps, List *req, WSPMachine *m) {
 		data = octstr_create("");
 		octstr_append_char(data, 0);
 		cap = wsp_cap_create(WSP_CAPS_PROTOCOL_OPTIONS, NULL, data);
-		list_append(caps, cap);
+		gwlist_append(caps, cap);
 	}
 
 	/* Accept any Method-MOR the client sent; if it sent none,
@@ -813,7 +874,7 @@ static void reply_known_capabilities(List *caps, List *req, WSPMachine *m) {
 		data = octstr_create("");
 		octstr_append_char(data, ui);
 		cap = wsp_cap_create(WSP_CAPS_METHOD_MOR, NULL, data);
-		list_append(caps, cap);
+		gwlist_append(caps, cap);
 	}
 
 	/* We will never send any Push requests because we don't support
@@ -826,7 +887,7 @@ static void reply_known_capabilities(List *caps, List *req, WSPMachine *m) {
 		data = octstr_create("");
 		octstr_append_char(data, m->MOR_push);
 		cap = wsp_cap_create(WSP_CAPS_PUSH_MOR, NULL, data);
-		list_append(caps, cap);
+		gwlist_append(caps, cap);
 	}
 
 	/* Supporting extended methods is up to the application layer,
@@ -849,12 +910,12 @@ static void refuse_unreplied_capabilities(List *caps, List *req) {
 	long i, len;
 	Capability *cap;
 
-	len = list_len(req);
+	len = gwlist_len(req);
 	for (i = 0; i < len; i++) {
-		cap = list_get(req, i);
+		cap = gwlist_get(req, i);
 		if (wsp_cap_count(caps, cap->id, cap->name) == 0) {
 			cap = wsp_cap_create(cap->id, cap->name, NULL);
-			list_append(caps, cap);
+			gwlist_append(caps, cap);
 		}
 	}
 }
@@ -898,12 +959,12 @@ static void strip_default_capabilities(List *caps, List *req) {
 	/* Hmm, this is an O(N*N) operation, which may be bad. */
 
 	i = 0;
-	while (i < list_len(caps)) {
-		cap = list_get(caps, i);
+	while (i < gwlist_len(caps)) {
+		cap = gwlist_get(caps, i);
 
 		count = wsp_cap_count(req, cap->id, cap->name);
 		if (count == 0 && is_default_cap(cap)) {
-			list_delete(caps, i, 1);
+			gwlist_delete(caps, i, 1);
 			wsp_cap_destroy(cap);
 		} else {
 			i++;
@@ -945,48 +1006,80 @@ static List *make_capabilities_reply(WSPMachine *m) {
 }
 
 
-static Octstr *make_connectreply_pdu(WSPMachine *m) {
-	WSP_PDU *pdu;
-	Octstr *os;
-	List *caps;
+static List *make_reply_headers(WSPMachine *m)
+{
+    List *headers;
+    Octstr *encoding_version;
+
+    /* Add all server wsp level hop-by-hop headers. Currently only 
+     * Encoding-Version, as defined by wsp, chapter 8.4.2.70. 
+     * What headers belong to which version is defined in appendix A,
+     * table 39.. 
+    encoding_version = request_version = NULL;
+     * Essentially, if the client sends us an Encoding-Version
+     * higher than ours (1.3) we send our version number to it,
+     * if it is lower, we left version number intact. */
+    /* First the case that we have no Encoding-Version header at all. 
+     * This case we must assume that the client supports version 1.2
+     * or lower. */
+
+    headers = http_create_empty_headers();
+    encoding_version = wsp_encoding_version_to_string(m->encoding_version);
+    http_header_add(headers, "Encoding-Version", octstr_get_cstr(encoding_version));
+    octstr_destroy(encoding_version);
+
+    return headers;
+}
+
+static Octstr *make_connectreply_pdu(WSPMachine *m) 
+{
+    WSP_PDU *pdu;
+    Octstr *os;
+    List *caps;
+    List *reply_headers;
 	
-	pdu = wsp_pdu_create(ConnectReply);
+    pdu = wsp_pdu_create(ConnectReply);
 
-	pdu->u.ConnectReply.sessionid = m->session_id;
+    pdu->u.ConnectReply.sessionid = m->session_id;
 
-	caps = make_capabilities_reply(m);
-	pdu->u.ConnectReply.capabilities = wsp_cap_pack_list(caps);
-	wsp_cap_destroy_list(caps);
-	pdu->u.ConnectReply.headers = NULL;
+    caps = make_capabilities_reply(m);
+    pdu->u.ConnectReply.capabilities = wsp_cap_pack_list(caps);
+    wsp_cap_destroy_list(caps);
+
+    reply_headers = make_reply_headers(m);
+    pdu->u.ConnectReply.headers = 
+        wsp_headers_pack(reply_headers, 0, m->encoding_version);
+    http_destroy_headers(reply_headers);
 	
-	os = wsp_pdu_pack(pdu);
-	wsp_pdu_destroy(pdu);
+    os = wsp_pdu_pack(pdu);
+    wsp_pdu_destroy(pdu);
 
-	return os;
+    return os;
 }
 
 
-static Octstr *make_resume_reply_pdu(WSPMachine *m, List *headers) {
-	WSP_PDU *pdu;
-	Octstr *os;
+static Octstr *make_resume_reply_pdu(WSPMachine *m, List *headers) 
+{
+    WSP_PDU *pdu;
+    Octstr *os;
 
-	pdu = wsp_pdu_create(Reply);
+    pdu = wsp_pdu_create(Reply);
 
-	/* Not specified for Resume replies */
-	pdu->u.Reply.status = wsp_convert_http_status_to_wsp_status(HTTP_OK);
-	if (headers == NULL) {
-		headers = http_create_empty_headers();
-		pdu->u.Reply.headers = wsp_headers_pack(headers, 1);
-		http_destroy_headers(headers);
-	} else {
-		pdu->u.Reply.headers = wsp_headers_pack(headers, 1);
-	}
-	pdu->u.Reply.data = octstr_create("");
+    /* Not specified for Resume replies */
+    pdu->u.Reply.status = wsp_convert_http_status_to_wsp_status(HTTP_OK);
+    if (headers == NULL) {
+        headers = http_create_empty_headers();
+        pdu->u.Reply.headers = wsp_headers_pack(headers, 1, m->encoding_version);
+        http_destroy_headers(headers);
+    } else {
+        pdu->u.Reply.headers = wsp_headers_pack(headers, 1, m->encoding_version);
+    }
+    pdu->u.Reply.data = octstr_create("");
 
-	os = wsp_pdu_pack(pdu);
-	wsp_pdu_destroy(pdu);
+    os = wsp_pdu_pack(pdu);
+    wsp_pdu_destroy(pdu);
 
-	return os;
+    return os;
 }
 
 static WSP_PDU *make_confirmedpush_pdu(WAPEvent *e)
@@ -1000,11 +1093,11 @@ static WSP_PDU *make_confirmedpush_pdu(WAPEvent *e)
  */
         if (e->u.S_ConfirmedPush_Req.push_headers == NULL) {
 	    headers = http_create_empty_headers();
-            pdu->u.ConfirmedPush.headers = wsp_headers_pack(headers, 1);
+            pdu->u.ConfirmedPush.headers = wsp_headers_pack(headers, 1, WSP_1_2);
             http_destroy_headers(headers);
         } else
             pdu->u.ConfirmedPush.headers = 
-                wsp_headers_pack(e->u.S_ConfirmedPush_Req.push_headers, 1);
+                wsp_headers_pack(e->u.S_ConfirmedPush_Req.push_headers, 1, WSP_1_2);
    
         if (e->u.S_ConfirmedPush_Req.push_body == NULL)
 	    pdu->u.ConfirmedPush.data = octstr_create("");
@@ -1026,11 +1119,11 @@ static WSP_PDU *make_push_pdu(WAPEvent *e)
  */
         if (e->u.S_Push_Req.push_headers == NULL) {
 	    headers = http_create_empty_headers();
-            pdu->u.Push.headers = wsp_headers_pack(headers, 1);
+            pdu->u.Push.headers = wsp_headers_pack(headers, 1, WSP_1_2);
             http_destroy_headers(headers);
         } else
             pdu->u.Push.headers = 
-                wsp_headers_pack(e->u.S_Push_Req.push_headers, 1);
+                wsp_headers_pack(e->u.S_Push_Req.push_headers, 1, WSP_1_2);
    
         if (e->u.S_Push_Req.push_body == NULL)
 	    pdu->u.Push.data = octstr_create("");
@@ -1075,12 +1168,12 @@ static int find_by_push_id(void *m_ptr, void *id_ptr) {
 }
 
 static WSPMethodMachine *find_method_machine(WSPMachine *sm, long id) {
-	return list_search(sm->methodmachines, &id, find_by_method_id);
+	return gwlist_search(sm->methodmachines, &id, find_by_method_id);
 }
 
 static WSPPushMachine *find_push_machine(WSPMachine *m, long id)
 {
-       return list_search(m->pushmachines, &id, find_by_push_id);
+       return gwlist_search(m->pushmachines, &id, find_by_push_id);
 }
 
 static int same_client(void *a, void *b) {
@@ -1098,19 +1191,19 @@ static void disconnect_other_sessions(WSPMachine *sm) {
 	WSPMachine *sm2;
 	long i;
 
-	old_sessions = list_search_all(session_machines, sm, same_client);
+	old_sessions = gwlist_search_all(session_machines, sm, same_client);
 	if (old_sessions == NULL)
 		return;
 
-	for (i = 0; i < list_len(old_sessions); i++) {
-		sm2 = list_get(old_sessions, i);
+	for (i = 0; i < gwlist_len(old_sessions); i++) {
+		sm2 = gwlist_get(old_sessions, i);
 		if (sm2 != sm) {
 			disconnect = wap_event_create(Disconnect_Event);
 			handle_session_event(sm2, disconnect, NULL);
 		}
 	}
 
-	list_destroy(old_sessions, NULL);
+	gwlist_destroy(old_sessions, NULL);
 }
 
 
@@ -1281,7 +1374,7 @@ static void release_holding_methods(WSPMachine *sm) {
 	List *holding;
 	long i, len;
 
-	holding = list_search_all(sm->methodmachines, NULL, method_is_holding);
+	holding = gwlist_search_all(sm->methodmachines, NULL, method_is_holding);
 	if (holding == NULL)
 		return;
 
@@ -1289,12 +1382,12 @@ static void release_holding_methods(WSPMachine *sm) {
 	 * destroy its event */
 	release = wap_event_create(Release_Event);
 
-	len = list_len(holding);
+	len = gwlist_len(holding);
 	for (i = 0; i < len; i++) {
-		msm = list_get(holding, i);
+		msm = gwlist_get(holding, i);
 		handle_method_event(sm, msm, release, NULL);
 	}
-	list_destroy(holding, NULL);
+	gwlist_destroy(holding, NULL);
 	wap_event_destroy(release);
 }
 
@@ -1309,15 +1402,16 @@ static void abort_methods(WSPMachine *sm, long reason) {
 
 	/* This loop goes backward because it has to deal with the
 	 * possibility of method machines disappearing after their event. */
-	len = list_len(sm->methodmachines);
+	len = gwlist_len(sm->methodmachines);
 	for (i = len - 1; i >= 0; i--) {
-		msm = list_get(sm->methodmachines, i);
+		msm = gwlist_get(sm->methodmachines, i);
 		handle_method_event(sm, msm, ab, NULL);
 	}
 
 	wap_event_destroy(ab);
 }
 
+
 static void abort_pushes(WSPMachine *sm, long reason)
 {
         WAPEvent *ab;
@@ -1327,9 +1421,9 @@ static void abort_pushes(WSPMachine *sm, long reason)
         ab = wap_event_create(Abort_Event);
 	ab->u.Abort_Event.reason = reason;
 
-        len = list_len(sm->pushmachines);
+        len = gwlist_len(sm->pushmachines);
 	for (i = len - 1; i >= 0; i--) {
-		psm = list_get(sm->pushmachines, i);
+		psm = gwlist_get(sm->pushmachines, i);
 		handle_push_event(sm, psm, ab);
 	}
 
@@ -1339,9 +1433,10 @@ static void abort_pushes(WSPMachine *sm, long reason)
 
 WSPMachine *find_session_machine_by_id (int id) {
 
-	return list_search(session_machines, &id, id_belongs_to_session);
+	return gwlist_search(session_machines, &id, id_belongs_to_session);
 }
 
+
 static int id_belongs_to_session (void *wsp_ptr, void *pid) {
 	WSPMachine *wsp;
 	int *id;
@@ -1354,10 +1449,57 @@ static int id_belongs_to_session (void *wsp_ptr, void *pid) {
 }
 
 
+static int wsp_encoding_string_to_version(Octstr *enc) 
+{
+    int v;
+    
+    /* default will be WSP 1.2, as defined by WAPWSP */
+    v = WSP_1_2;    
 
+    if (octstr_compare(enc, octstr_imm("1.1")) == 0) {
+        v = WSP_1_1;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.2")) == 0) {
+        v = WSP_1_2;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.3")) == 0) {
+        v = WSP_1_3;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.4")) == 0) {
+        v = WSP_1_4;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.5")) == 0) {
+        v = WSP_1_5;
+    }
 
+    return v;
+}
 
-
-
-
+static Octstr *wsp_encoding_version_to_string(int version) 
+{
+    Octstr *os;
+    
+    switch (version) {
+        case WSP_1_1:
+            os = octstr_create("1.1");
+            break;
+        case WSP_1_2:
+            os = octstr_create("1.2");
+            break;
+        case WSP_1_3:
+            os = octstr_create("1.3");
+            break;
+        case WSP_1_4:
+            os = octstr_create("1.4");
+            break;
+        case WSP_1_5:
+            os = octstr_create("1.5");
+            break;
+        default:
+            os = octstr_create("1.2");
+            break;
+    }
+    
+    return os;
+}
 
